import os
import argparse
from Bio import AlignIO


PB_usage = '''
========================================== PB example commands ==========================================

# Dependency: mpirun, pb_mpi and readpb_mpi (from PhyloBayes-MPI)

export OMPI_MCA_btl=^openib
TreeSAK PB -i in.phylip -p best20pb -o best20pb -t 52 
TreeSAK PB -i in.phylip -p best20pb -o best20pb -t 52 -n 1
TreeSAK PB -i in.phylip -p worst20pb -o worst20pb -t 52 

# Notes:
1. This is a wrapper for: mpirun -np 12 pb_mpi -d in.phylip -cat -gtr -x 10 -1 -dgam 4 -s chain_name
2. Input MSA need to be in phylip format.
3. To stop a chain, just open the <chain_name>.run ﬁle and replace the 1 by a 0 (echo 0 > <chain_name>.run).
4. Be careful not to restart an already running chain.
5. You can stop a chain and restart it under a diﬀerent degree of parallelization.
6. Generally, PhyloBayes provides good results for a total number of points of 10000-30000.
7. Results can be assessed with bpcomp and tracecomp

*  Settings used by Nina Dombrowski: -cat -gtr -x 10 -1 -dgam 4
   For each marker protein family, four parallel chains were run until convergence was reached, unless stated 
   otherwise (maxdiff < 0.3; settings: bpcomp -x 25_burnin chain1 chain2 chain3 chain4). Additionally, we 
   checked for the minimum effective size using tracecomp (minimum effective size > 50; settings: -x 25_burnin 
   chain1 chain2 chain3 chain4).
   
*  Settings used by Fan Lu:
   Four chains were run for each consensus tree, and for each chain over 15,000 cycles (5,000 burn-in) 
   were conducted, until a maxdiff value lower than 0.3 was reached. Otherwise, non-converged chains were 
   continually run to over 20,000 cycles. Posterior predictive tests were conducted using PhyloBayes MPI 
   with the ‘readpb_mpi -x 5000 50 -allppred’ command.

=========================================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def fa2phy(fasta_in, phy_out):

    alignment = AlignIO.read(fasta_in, 'fasta')

    max_seq_id_len = 0
    for each_seq in alignment:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len

    with open(phy_out, 'w') as msa_out_handle:
        msa_out_handle.write('%s %s\n' % (len(alignment), alignment.get_alignment_length()))
        for each_seq in alignment:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))


def PB(args):

    msa_in          = args['i']
    op_dir          = args['o']
    op_prefix       = args['p']
    fa_to_plp       = args['fa2plp']
    num_of_threads  = args['t']
    num_of_chains   = args['n']
    force_overwrite = args['f']

    ####################################################################################################################

    msa_in_name, msa_in_path, msa_in_base, msa_in_ext = sep_path_basename_ext(msa_in)

    settings_dombrowski = '-cat -gtr -x 10 -1 -dgam 4'
    setting_to_use  = settings_dombrowski
    msa_in_plp      = '%s/%s.phylip'        % (op_dir, msa_in_base)
    cmd_txt         = '%s/%s_cmds.txt'      % (op_dir, msa_in_base)

    ####################################################################################################################

    # create output dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output folder already exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    # fa_to_phylip
    msa_to_use = msa_in
    if fa_to_plp is True:
        fa2phy(msa_in, msa_in_plp)
        msa_to_use = msa_in_plp

    chain_name_list = []
    pb_mpi_cmd_list = []
    if num_of_chains == 1:
        pb_mpi_cmd = 'export OMPI_MCA_btl=^openib; mpirun -np %s pb_mpi -d %s %s -s %s/%s' % (num_of_threads, msa_to_use, setting_to_use, op_dir, op_prefix)
        chain_name_list.append('%s/%s' % (op_dir, op_prefix))
        pb_mpi_cmd_list.append(pb_mpi_cmd)
    else:
        for chain_index in range(1, (num_of_chains + 1)):
            current_wd = '%s/%s_chain%s' % (op_dir, op_prefix, chain_index)
            os.mkdir(current_wd)
            pb_mpi_cmd = 'export OMPI_MCA_btl=^openib; mpirun -np %s pb_mpi -d %s %s -s %s/%s_chain%s' % (num_of_threads, msa_to_use, setting_to_use, current_wd, op_prefix, chain_index)
            chain_name_list.append('%s/%s_chain%s' % (current_wd, op_prefix, chain_index))
            pb_mpi_cmd_list.append(pb_mpi_cmd)

    # write out commands
    cmd_txt_handle = open(cmd_txt, 'w')
    cmd_txt_handle.write('# To run pb_mpi\n')
    for cmd in pb_mpi_cmd_list:
        cmd_txt_handle.write(cmd + '\n')

    cmd_txt_handle.write('\n# To restart a terminated run (e.g., due to walltime limitation)\n')
    for each_chain in chain_name_list:
        cmd_txt_handle.write('export OMPI_MCA_btl=^openib; mpirun -np %s pb_mpi %s\n' % (num_of_threads, each_chain))
    cmd_txt_handle.close()

    # assess the results
    if num_of_chains > 1:
        readpb_cmd = 'export OMPI_MCA_btl=^openib; bpcomp -x 1000 10 %s' % (' '.join(chain_name_list))
        bpcomp_cmd = 'export OMPI_MCA_btl=^openib; tracecomp -x 1000 %s' % (' '.join(chain_name_list))
        cmd_txt_handle = open(cmd_txt, 'a')
        cmd_txt_handle.write('\n# You may want to use the following commands to assess the results:\n')
        cmd_txt_handle.write(readpb_cmd + '\n')
        cmd_txt_handle.write(bpcomp_cmd + '\n')
        cmd_txt_handle.close()

    print('Commands exported to %s' % cmd_txt)
    print('Done!')


if __name__ == '__main__':

    PB_parser = argparse.ArgumentParser()
    PB_parser.add_argument('-i',       required=True,                          help='input MSA file')
    PB_parser.add_argument('-o',       required=True,                          help='output directory')
    PB_parser.add_argument('-p',       required=True,                          help='output prefix')
    PB_parser.add_argument('-fa2plp',  required=False, action="store_true",    help='convert MSA format from fasta to phylip')
    PB_parser.add_argument('-n',       required=False, type=int, default=4,    help='number of chains, default: 4')
    PB_parser.add_argument('-t',       required=False, type=int, default=48,   help='num of cores per mpirun, default: 48')
    PB_parser.add_argument('-f',       required=False, action="store_true",    help='force overwrite')
    args = vars(PB_parser.parse_args())
    PB(args)



import random
import dendropy
import argparse
from ete3 import Tree


RootTree_usage = '''
====================== RootTree example commands ======================

TreeSAK RootTree -i input.tree -og outgroup_genomes.txt -o rooted.tree

=======================================================================
'''


def root_with_outgroup(input_tree, out_group_list, add_root_branch, tree_file_rooted):

    """
    Reroot the tree using the given outgroup.
    modified based on: https://github.com/Ecogenomics/GTDBTk/blob/master/gtdbtk/reroot_tree.py

    input_tree:  File containing Newick tree to rerooted.
    output_tree: Name of file for rerooted tree.
    outgroup:    Labels of taxa in outgroup.
    """

    tree = dendropy.Tree.get_from_path(input_tree, schema='newick', rooting='force-rooted', preserve_underscores=True)

    outgroup_in_tree = set()
    ingroup_leaves = set()
    for n in tree.leaf_node_iter():
        if n.taxon.label in out_group_list:
            outgroup_in_tree.add(n.taxon)
        else:
            ingroup_leaves.add(n)

    # Since finding the MRCA is a rooted tree operation, the tree is first rerooted on an ingroup taxa. This
    # ensures the MRCA of the outgroup can be identified so long as the outgroup is monophyletic. If the
    # outgroup is polyphyletic trying to root on it is ill-defined. To try and pick a "good" root for
    # polyphyletic outgroups, random ingroup taxa are selected until two of them give the same size
    # lineage. This will, likely, be the smallest bipartition possible for the given outgroup though
    # this is not guaranteed.

    mrca = tree.mrca(taxa=outgroup_in_tree)
    mrca_leaves = len(mrca.leaf_nodes())
    while True:
        rnd_ingroup = random.sample(list(ingroup_leaves), 1)[0]
        tree.reroot_at_edge(rnd_ingroup.edge, length1=0.5 * rnd_ingroup.edge_length, length2=0.5 * rnd_ingroup.edge_length)
        mrca = tree.mrca(taxa=outgroup_in_tree)
        if len(mrca.leaf_nodes()) == mrca_leaves:
            break
        mrca_leaves = len(mrca.leaf_nodes())

    if mrca.edge_length is not None:
        tree.reroot_at_edge(mrca.edge, length1=0.5 * mrca.edge_length, length2=0.5 * mrca.edge_length)

        # tree.write_to_path(tree_file_rooted, schema='newick', suppress_rooting=True, unquoted_underscores=True)
        tree_out_string = tree.as_string(schema='newick', suppress_rooting=True, unquoted_underscores=True)
        tree_out_string = tree_out_string.replace("'", "")

        # add the root bar
        if add_root_branch is True:
            tree_out_string = '(' + tree_out_string
            tree_out_string = tree_out_string.replace(');', '):0.02);')

        # write out tree string
        tree_file_rooted_handle = open(tree_file_rooted, 'w')
        tree_file_rooted_handle.write(tree_out_string)
        tree_file_rooted_handle.close()


def RootTree(args):

    tree_file           = args['i']
    out_group_txt       = args['og']
    tree_file_rooted    = args['o']
    tree_fmt            = args['fmt']
    add_root_branch     = args['add_root']

    out_group_set = set()
    for each_og in open(out_group_txt):
        out_group_set.add(each_og.strip())

    # tre = Tree(tree_file, format=tree_fmt)
    # out_group_lca = tre.get_common_ancestor(out_group_set)
    # tre.set_outgroup(out_group_lca)
    # tre.write(outfile=tree_file_rooted, format=tree_fmt)

    root_with_outgroup(tree_file, out_group_set, add_root_branch, tree_file_rooted)


if __name__ == '__main__':

    RootTree_parser = argparse.ArgumentParser()
    RootTree_parser.add_argument('-i',          required=True,                          help='input tree')
    RootTree_parser.add_argument('-og',         required=True,                          help='out group leaves')
    RootTree_parser.add_argument('-o',          required=True,                          help='output tree')
    RootTree_parser.add_argument('-add_root',   required=False, action='store_true',    help='add the root branch')
    RootTree_parser.add_argument('-fmt',        required=False, default=1, type=int,    help='tree format, default: 1')
    args = vars(RootTree_parser.parse_args())
    RootTree(args)



import os
import argparse


ALE7_usage = '''
=============================== ALE7 example commands ===============================

# get presence/absence of interested functions in reconstructed ancestral genomes

TreeSAK ALE7 -6 ALE6_op_dir -fun ko.txt -node internal_node.txt -o Fun_PA.txt
TreeSAK ALE7 -6 ALE6_op_dir -fun K01995,K01995 -node 359,466,470 -o Fun_PA.txt
TreeSAK ALE7 -6 ALE6_op_dir -fun arCOG07811,K01995 -node 359,466,470 -o Fun_PA.txt

# needed input files:
-6: annotation_COG/annotation_KEGG

=====================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def ALE7(args):

    ale6_op_dir        = args['6']
    interested_fun_txt = args['fun']
    interested_gnm_txt = args['node']
    op_txt             = args['o']

    op_txt_name, op_txt_path, op_txt_base, op_txt_ext = sep_path_basename_ext(op_txt)
    op_txt_pct = '%s/%s_percentage.%s' % (op_txt_path, op_txt_base, op_txt_ext)

    if os.path.isdir(ale6_op_dir) is False:
        print('%s not found, program exited!' % ale6_op_dir)
        exit()

    interested_fun_set = set()
    if os.path.isfile(interested_fun_txt) is False:
        if ',' in interested_fun_txt:
            interested_fun_set = interested_fun_txt.split(',')
        else:
            interested_fun_set.add(interested_fun_txt)
    else:
        for each_fun in open(interested_fun_txt):
            interested_fun_set.add(each_fun.strip().split()[0])

    node_desc_dict = dict()
    interested_node_set = set()
    if os.path.isfile(interested_gnm_txt) is False:
        if ',' in interested_gnm_txt:
            interested_node_set = interested_gnm_txt.split(',')
        else:
            interested_node_set.add(interested_gnm_txt)
    else:
        for each_node in open(interested_gnm_txt):
            each_node_split = each_node.strip().split()
            node_id = each_node_split[0]
            interested_node_set.add(each_node.strip().split()[0])
            if len(each_node_split) >= 2:
                node_desc = each_node_split[1]
                node_desc_dict[node_id] = node_desc

    interested_fun_list_sorted = sorted(list(interested_fun_set))
    interested_gnm_list_sorted = sorted(list(interested_node_set))

    op_txt_handle = open(op_txt, 'w')
    op_txt_pct_handle = open(op_txt_pct, 'w')
    if len(node_desc_dict) == 0:
        op_txt_handle.write('ID\t' + '\t'.join(interested_fun_list_sorted) + '\n')
        op_txt_pct_handle.write('ID\tPercentage\n')
    else:
        op_txt_handle.write('ID\tDescription\t' + '\t'.join(interested_fun_list_sorted) + '\n')
        op_txt_pct_handle.write('ID\tDescription\tPercentage\n')
    for each_node in interested_gnm_list_sorted:

        node_desc            = node_desc_dict.get(each_node, 'na')
        node_annotation_cog  = '%s/annotation_COG/%s_COG.txt'   % (ale6_op_dir, each_node)
        node_annotation_kegg = '%s/annotation_KEGG/%s_KEGG.txt' % (ale6_op_dir, each_node)

        node_fun_set = set()
        if os.path.isfile(node_annotation_cog):
            for each_line in open(node_annotation_cog):
                node_fun_set.add(each_line.strip().split()[0])
        if os.path.isfile(node_annotation_kegg):
            for each_line in open(node_annotation_kegg):
                node_fun_set.add(each_line.strip().split()[0])

        fun_pa_list = []
        for each_fun in interested_fun_list_sorted:
            if each_fun in node_fun_set:
                fun_pa_list.append('1')
            else:
                fun_pa_list.append('0')

        fun_presence_pct = fun_pa_list.count('1')*100/len(fun_pa_list)
        fun_presence_pct = float("{0:.2f}".format(fun_presence_pct))
        if len(node_desc_dict) == 0:
            op_txt_handle.write('%s\t%s\n' % (each_node, '\t'.join(fun_pa_list)))
            op_txt_pct_handle.write('%s\t%s\n' % (each_node, fun_presence_pct))
        else:
            op_txt_handle.write('%s\t%s\t%s\n' % (each_node, node_desc, '\t'.join(fun_pa_list)))
            op_txt_pct_handle.write('%s\t%s\t%s\n' % (each_node, node_desc, fun_presence_pct))
    op_txt_handle.close()
    op_txt_pct_handle.close()

    print('Results exported to %s and %s' % (op_txt, op_txt_pct))
    print('Done!')


if __name__ == '__main__':

    ALE7_parser = argparse.ArgumentParser()
    ALE7_parser.add_argument('-6',      required=True,                          help='ALE6 output directory')
    ALE7_parser.add_argument('-fun',    required=True,                          help='interested functions')
    ALE7_parser.add_argument('-node',   required=True,                          help='interested internal nodes')
    ALE7_parser.add_argument('-o',      required=True,                          help='output directory')
    args = vars(ALE7_parser.parse_args())
    ALE7(args)



import os
import argparse
from ete3 import Tree


FigTree_usage = '''
====================== FigTree example commands ======================

TreeSAK FigTree -h

======================================================================
'''


def FigTree(args):

    input_txt_file  = args['i']
    op_txt          = args['o']

    if os.path.isfile(input_txt_file) is False:
        print('Metadata file not found, program exited!')
        exit()


if __name__ == '__main__':

    FigTree_parser = argparse.ArgumentParser(usage=FigTree_usage)
    FigTree_parser.add_argument('-i',     required=True,                         help='input metadata')
    FigTree_parser.add_argument('-tree',  required=False, default=None,          help='gene id, in tree file')
    FigTree_parser.add_argument('-txt',   required=False, default=None,          help='gene id, in txt file')
    FigTree_parser.add_argument('-o',     required=True,                         help='output metadata')
    FigTree_parser.add_argument('-na',    required=False, action='store_true',   help='include leaves with na values')
    args = vars(FigTree_parser.parse_args())
    FigTree(args)



import os
import argparse


GTDB_tree_usage = '''
======================== GTDB_tree example command ========================

export GTDBTK_DATA_PATH=/scratch/PI/boqianpy/Database/gtdb_r220/release220
TreeSAK GTDB_tree -p Demo -i gnm_folder -x fa -t 12

# This is a wrapper for the following commands
gtdbtk identify --genome_dir gnm_folder -x fa --out_dir op_dir --cpus 12
gtdbtk align --identify_dir Demo_op_dir --out_dir op_dir --cpus 12
gtdbtk infer --msa_file Demo_op_dir/align/gtdbtk.bac120.user_msa.fasta.gz --out_dir op_dir --cpus 12 --prefix Demo_bac120
gtdbtk infer --msa_file Demo_op_dir/align/gtdbtk.ar53.user_msa.fasta.gz --out_dir op_dir --cpus 12 --prefix Demo_ar53

===========================================================================
'''

def GTDB_tree(args):

    input_gnm_dir       = args['i']
    output_prefix       = args['p']
    file_extension      = args['x']
    num_threads         = args['t']

    output_dir          = '%s_GTDB_tree'                                                            % output_prefix
    msa_bac120_gz       = '%s/align/gtdbtk.bac120.user_msa.fasta.gz'                                % output_dir
    msa_bac120          = '%s/align/gtdbtk.bac120.user_msa.fasta'                                   % output_dir
    msa_ar53_gz         = '%s/align/gtdbtk.ar53.user_msa.fasta.gz'                                  % output_dir
    msa_ar53            = '%s/align/gtdbtk.ar53.user_msa.fasta'                                     % output_dir

    cmd_identify        = 'gtdbtk identify --genome_dir %s -x %s --out_dir %s --cpus %s --write_single_copy_genes'  % (input_gnm_dir, file_extension, output_dir, num_threads)
    cmd_align           = 'gtdbtk align --identify_dir %s --out_dir %s --cpus %s'                                   % (output_dir, output_dir, num_threads)
    cmd_gunzip_bac120   = 'gunzip %s'                                                                               % msa_bac120_gz
    cmd_gunzip_ar53     = 'gunzip %s'                                                                               % msa_ar53_gz
    cmd_infer_bac120    = 'gtdbtk infer --msa_file %s --out_dir %s --cpus %s --prefix %s_bac120'                    % (msa_bac120, output_dir, num_threads, output_prefix)
    cmd_infer_ar53      = 'gtdbtk infer --msa_file %s --out_dir %s --cpus %s --prefix %s_ar53'                      % (msa_ar53, output_dir, num_threads, output_prefix)

    print(cmd_identify)
    os.system(cmd_identify)
    print(cmd_align)
    os.system(cmd_align)

    if os.path.isfile(msa_bac120_gz):
        print(cmd_gunzip_bac120)
        os.system(cmd_gunzip_bac120)
        print(cmd_infer_bac120)
        os.system(cmd_infer_bac120)

    if os.path.isfile(msa_ar53_gz):
        print(cmd_gunzip_ar53)
        os.system(cmd_gunzip_ar53)
        print(cmd_infer_ar53)
        os.system(cmd_infer_ar53)

    inferred_bac120_tree = '%s/%s_bac120.unrooted.tree' % (output_dir, output_prefix)
    inferred_ar53_tree   = '%s/%s_ar53.unrooted.tree'   % (output_dir, output_prefix)

    if os.path.isfile(inferred_bac120_tree):
        print('Inferred bacterial tree:\t%s' % inferred_bac120_tree)
    if os.path.isfile(inferred_ar53_tree):
        print('Inferred archaeal tree:\t%s' % inferred_ar53_tree)

    print('Done!')


if __name__ == '__main__':

    GTDB_tree_parser = argparse.ArgumentParser(usage=GTDB_tree_usage)
    GTDB_tree_parser.add_argument('-p',               required=True,                            help='output prefix')
    GTDB_tree_parser.add_argument('-i',               required=True,                            help='genome folder')
    GTDB_tree_parser.add_argument('-x',               required=True,                            help='genome file extension')
    GTDB_tree_parser.add_argument('-t',               required=False, type=int, default=1,      help='number of threads')
    args = vars(GTDB_tree_parser.parse_args())
    GTDB_tree(args)



import os
import argparse
import itertools
from ete3 import Tree
from Bio import AlignIO


Dating_usage = '''
============================= Dating example commands =============================

# example commands
TreeSAK Dating_ss -deltall DeltaLL_stdout.txt -aod s11_marker_sets_by_DeltaLL -o s12_dating_wd -c 25-50-75-100 -mmn 20 -f

===================================================================================
'''


def sep_path_basename_ext(file_in):
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'
    file_basename, file_extension = os.path.splitext(file_name)
    return file_path, file_basename, file_extension


def submit_js(js):
    current_wd = os.getcwd()
    js_path, js_basename, js_ext = sep_path_basename_ext(js)
    os.chdir(js_path)
    os.system('qsub %s%s' % (js_basename, js_ext))
    os.chdir(current_wd)


def root_with_out_group(tree_file, out_group_txt, tree_file_rooted):

    out_group_set = set()
    for each_og in open(out_group_txt):
        out_group_set.add(each_og.strip())

    tre = Tree(tree_file, format=1)
    out_group_lca = tre.get_common_ancestor(out_group_set)
    tre.set_outgroup(out_group_lca)
    tre.write(outfile=tree_file_rooted)


def replace_clades(main_tree, sub_tree, tree_out, quote_node_name):

    tre_sub = Tree(sub_tree, format=1, quoted_node_names=quote_node_name)
    subtree_leaf_name_list = tre_sub.get_leaf_names()
    tre_main = Tree(main_tree)
    lca = tre_main.get_common_ancestor(subtree_leaf_name_list)

    if len(lca.get_leaf_names()) != len(subtree_leaf_name_list):
        print('LCA of subtree leaves in main tree contain extra leaves, program exited!')
        exit()

    lca_p = lca.up
    lca_p.remove_child(lca)
    lca_p.add_child(tre_sub)
    tre_main.write(outfile=tree_out, format=8, quoted_node_names=quote_node_name)


def prep_mcmctree_ctl(ctl_para_dict, mcmctree_ctl_file):

    with open(mcmctree_ctl_file, 'w') as ctl_file_handle:
        ctl_file_handle.write('      finetune = %s\n' % ctl_para_dict.get('seed', '-1'))
        ctl_file_handle.write('       seqfile = %s\n' % ctl_para_dict['seqfile'])
        ctl_file_handle.write('      treefile = %s\n' % ctl_para_dict['treefile'])
        ctl_file_handle.write('      mcmcfile = %s\n' % ctl_para_dict['mcmcfile'])
        ctl_file_handle.write('       outfile = %s\n' % ctl_para_dict['outfile'])
        ctl_file_handle.write('         ndata = %s\n'                                                                           % ctl_para_dict.get('ndata',        1))
        ctl_file_handle.write('       seqtype = %s    	* 0: nucleotides; 1:codons; 2:AAs\n'                                    % ctl_para_dict['seqtype'])
        ctl_file_handle.write('       usedata = %s    	* 0: no data; 1:seq like; 2:normal approximation; 3:out.BV (in.BV)\n'   % ctl_para_dict['usedata'])
        ctl_file_handle.write('         clock = %s    	* 1: global clock; 2: independent rates; 3: correlated rates\n'         % ctl_para_dict['clock'])
        ctl_file_handle.write('       RootAge = %s      * safe constraint on root age, used if no fossil for root.\n'           % ctl_para_dict.get('RootAge',      '<1.0'))
        ctl_file_handle.write('         model = %s    	* 0:JC69, 1:K80, 2:F81, 3:F84, 4:HKY85\n'                               % ctl_para_dict.get('model',        0))
        ctl_file_handle.write('         alpha = %s  	* alpha for gamma rates at sites\n'                                     % ctl_para_dict.get('alpha',        0.5))
        ctl_file_handle.write('         ncatG = %s    	* No. categories in discrete gamma\n'                                   % ctl_para_dict.get('ncatG',        4))
        ctl_file_handle.write('     cleandata = %s    	* remove sites with ambiguity data (1:yes, 0:no)?\n'                    % ctl_para_dict.get('cleandata',    0))
        ctl_file_handle.write('       BDparas = %s      * birth, death, sampling\n'                                             % ctl_para_dict.get('BDparas',      '1 1 0.1'))
        ctl_file_handle.write('   kappa_gamma = %s      * gamma prior for kappa\n'                                              % ctl_para_dict.get('kappa_gamma',  '6 2'))
        ctl_file_handle.write('   alpha_gamma = %s      * gamma prior for alpha\n'                                              % ctl_para_dict.get('alpha_gamma',  '1 1'))
        ctl_file_handle.write('   rgene_gamma = %s      * gammaDir prior for rate for genes\n'                                  % ctl_para_dict.get('rgene_gamma',  '1 50 1'))
        ctl_file_handle.write('  sigma2_gamma = %s      * gammaDir prior for sigma^2     (for clock=2 or 3)\n'                  % ctl_para_dict.get('sigma2_gamma', '1 10 1'))
        ctl_file_handle.write('      finetune = %s      * auto (0 or 1): times, musigma2, rates, mixing, paras, FossilErr\n'    % ctl_para_dict.get('finetune',     '1: .1 .1 .1 .1 .1 .1'))
        ctl_file_handle.write('         print = %s      * 0: no mcmc sample; 1: everything except branch rates 2: everything\n' % ctl_para_dict.get('print',        1))
        ctl_file_handle.write('        burnin = %s\n'                                                                           % ctl_para_dict.get('burnin',       50000))
        ctl_file_handle.write('      sampfreq = %s\n'                                                                           % ctl_para_dict.get('sampfreq',     5))
        ctl_file_handle.write('       nsample = %s\n'                                                                           % ctl_para_dict.get('nsample',      150000))


def get_parameter_combinations(para_to_test_dict):

    para_lol_name = []
    para_lol_value = []
    para_lol_name_with_value = []
    for each_para in sorted(list(para_to_test_dict.keys())):
        para_setting_list_name = []
        para_setting_list_value = []
        para_setting_list_name_with_value = []
        for each_setting in sorted(para_to_test_dict[each_para]):
            name_str = ('%s%s' % (each_para, each_setting)).replace(' ', '_')
            para_setting_list_name.append(each_para)
            para_setting_list_value.append(each_setting)
            para_setting_list_name_with_value.append(name_str)
        para_lol_name.append(para_setting_list_name)
        para_lol_value.append(para_setting_list_value)
        para_lol_name_with_value.append(para_setting_list_name_with_value)

    all_combination_list_name = [p for p in itertools.product(*para_lol_name)]
    all_combination_list_value = [p for p in itertools.product(*para_lol_value)]
    all_combination_list_name_with_value = [p for p in itertools.product(*para_lol_name_with_value)]
    all_combination_list_name_with_value_str = ['_'.join(i) for i in all_combination_list_name_with_value]

    para_dod = dict()
    element_index = 0
    for each_combination in all_combination_list_name_with_value_str:
        current_name_list   = all_combination_list_name[element_index]
        current_value_list  = all_combination_list_value[element_index]
        current_para_dict = dict()
        for key, value in zip(current_name_list, current_value_list):
            current_para_dict[key] = value
        para_dod[each_combination] = current_para_dict
        element_index += 1

    return para_dod


def fa2phy(fasta_in, phy_out):

    alignment = AlignIO.read(fasta_in, 'fasta')

    max_seq_id_len = 0
    for each_seq in alignment:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len

    with open(phy_out, 'w') as msa_out_handle:
        msa_out_handle.write('%s %s\n' % (len(alignment), alignment.get_alignment_length()))
        for each_seq in alignment:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))


def dating_ss(args):

    deltall_stdout_txt      = args['deltall']
    aod                     = args['aod']
    out_group_txt           = args['og']
    eu_tree                 = args['eu']
    op_dir                  = args['o']
    deltall_keep_pct_str    = args['c']
    min_marker_num          = args['mmn']
    force_overwrite         = args['f']
    root_age                = args['ra']
    submit_job              = args['qsub']
    para_to_test            = args['to_test']
    js_cpu_num              = 1
    quote_node_name         = False

    para_to_test_dict = dict()
    for each_para in open(para_to_test):
        each_para_split = each_para.strip().split()
        para_list = each_para_split[1].split(',')
        para_to_test_dict[each_para_split[0]] = para_list
    print('Parameters to test: %s' % para_to_test_dict)

    if os.path.isfile(eu_tree) is False:
        print('%s not found, program exited!' % eu_tree)
        exit()

    deltall_keep_pct_list = [int(i) for i in deltall_keep_pct_str.split('-')]
    deltall_stdout_path, deltall_stdout_basename, deltall_stdout_ext = sep_path_basename_ext(deltall_stdout_txt)

    # create dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    # read in deltall_stdout_txt
    deltall_op_dict = dict()
    for each_line in open(deltall_stdout_txt):
        if not ((each_line.startswith('WARNING:')) or (each_line.startswith('awk:'))):
            each_line_split = each_line.strip().split('\t')
            marker_id = each_line_split[0]
            value = float(each_line_split[1])
            if marker_id not in deltall_op_dict:
                deltall_op_dict[marker_id] = [value]
            else:
                deltall_op_dict[marker_id].append(value)

    # assigned score to marker
    metric_1_dict = dict()
    metric_2_dict = dict()
    for each_marker in deltall_op_dict:
        metric_1_value = float("{0:.2f}".format(deltall_op_dict[each_marker][0]))
        metric_2_value = float("{0:.2f}".format(deltall_op_dict[each_marker][1]))
        metric_1_dict[each_marker] = metric_1_value
        metric_2_dict[each_marker] = metric_2_value

    metric_1_dict_sorted = {k: v for k, v in sorted(metric_1_dict.items(), key=lambda item: item[1])[::-1]}
    metric_2_dict_sorted = {k: v for k, v in sorted(metric_2_dict.items(), key=lambda item: item[1])}

    metric_1_score_dict = dict()
    metric_1_score = 1
    for each_marker_1 in metric_1_dict_sorted:
        metric_1_score_dict[each_marker_1] = metric_1_score
        metric_1_score += 1

    metric_2_score_dict = dict()
    metric_2_score = 1
    for each_marker_2 in metric_2_dict_sorted:
        metric_2_score_dict[each_marker_2] = metric_2_score
        metric_2_score += 1

    overall_score_dict = dict()
    for each_marker in deltall_op_dict:
        metric_score_1 = metric_1_score_dict[each_marker]
        metric_score_2 = metric_2_score_dict[each_marker]
        metric_score_overall = metric_score_1 + metric_score_2
        overall_score_dict[each_marker] = metric_score_overall
    marker_list_sorted_by_deltall = [k for k, v in sorted(overall_score_dict.items(), key=lambda item: item[1])]

    # get qualified marker list
    for each_keep_pct in deltall_keep_pct_list:
        marker_num_to_keep = round(len(marker_list_sorted_by_deltall)*each_keep_pct/100)

        if marker_num_to_keep < min_marker_num:
            print('Ignored DeltaLL cutoff at %s , the number of qualified markers (%s) less than %s' % (each_keep_pct, marker_num_to_keep, min_marker_num))
        else:
            prefix_base                              = '%s_DeltaLL_%s'                                          % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            aln_concatenated                         = '%s_DeltaLL_%s_concatenated.phy'                         % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            aln_concatenated_in_aod_wd_fasta         = '%s_DeltaLL_%s_concatenated.phy.fasta'                   % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            c60_tree_file_rooted_with_time_final     = '%s_DeltaLL_%s_rooted_with_time_final.treefile'          % (deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            pwd_c60_tree_file                        = '%s/%s_DeltaLL_%s_iqtree_C60_PMSF/concatenated.treefile' % (aod, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            pwd_c60_tree_file_renamed                = '%s/%s_DeltaLL_%s_raw.treefile'                          % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            pwd_c60_tree_file_rooted                 = '%s/%s_DeltaLL_%s_rooted.treefile'                       % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            pwd_c60_tree_file_rooted_with_time       = '%s/%s_DeltaLL_%s_rooted_with_time.treefile'             % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct)
            pwd_aln_concatenated_in_aod_wd_fasta     = '%s/%s'                                                  % (aod, aln_concatenated_in_aod_wd_fasta)
            pwd_aln_concatenated_in_op_wd_phylip     = '%s/%s'                                                  % (op_dir, aln_concatenated)
            pwd_c60_tree_file_rooted_with_time_final = '%s/%s'                                                  % (op_dir, c60_tree_file_rooted_with_time_final)
            get_BV_wd                                = '%s/%s_get_BV_wd'                                        % (op_dir, prefix_base)
            pwd_aln_concatenated_in_bv_wd_phylip     = '%s/%s'                                                  % (get_BV_wd, aln_concatenated)

            fa2phy(pwd_aln_concatenated_in_aod_wd_fasta, pwd_aln_concatenated_in_op_wd_phylip)
            os.system('cp %s %s'  % (pwd_c60_tree_file, pwd_c60_tree_file_renamed))

            # root genome tree with outgroup
            root_with_out_group(pwd_c60_tree_file_renamed, out_group_txt, pwd_c60_tree_file_rooted)

            # add time constraints
            replace_clades(pwd_c60_tree_file_rooted, eu_tree, pwd_c60_tree_file_rooted_with_time, quote_node_name)

            # remove "NoName" from the rooted tree with time constraints
            tree_str = open(pwd_c60_tree_file_rooted_with_time).readline().strip().replace('NoName', '')

            # add root age
            tree_str = tree_str.replace(';', '<%s;' % root_age)
            tre_object = Tree(tree_str, format=8, quoted_node_names=quote_node_name)
            with open(pwd_c60_tree_file_rooted_with_time_final, 'w') as pwd_c60_tree_file_rooted_with_time_final_hanlde:
                pwd_c60_tree_file_rooted_with_time_final_hanlde.write('%s\t1\n' % len(tre_object.get_leaf_names()))
                pwd_c60_tree_file_rooted_with_time_final_hanlde.write(tree_str.replace('""', '') + '\n')
                #pwd_c60_tree_file_rooted_with_time_final_hanlde.write(tree_str + '\n')

            # rm tmp tree files
            os.system('rm %s' % pwd_c60_tree_file_renamed)
            os.system('rm %s' % pwd_c60_tree_file_rooted)
            os.system('rm %s' % pwd_c60_tree_file_rooted_with_time)

            # get BV file
            os.mkdir(get_BV_wd)
            fa2phy(pwd_aln_concatenated_in_aod_wd_fasta, pwd_aln_concatenated_in_bv_wd_phylip)  # sequence in phylip format need to be in one line
            os.system('cp %s %s/' % (pwd_c60_tree_file_rooted_with_time_final, get_BV_wd))

            get_BV_js                       = '%s/%s_get_BV.sh'         % (op_dir, prefix_base)
            get_BV_mcmctree_ctl             = '%s_get_BV_mcmctree.ctl'  % (prefix_base)
            pwd_get_BV_mcmctree_ctl         = '%s/%s'                   % (get_BV_wd, get_BV_mcmctree_ctl)

            get_BV_para_dict = dict()
            get_BV_para_dict['seqfile']     = aln_concatenated
            get_BV_para_dict['treefile']    = c60_tree_file_rooted_with_time_final
            get_BV_para_dict['mcmcfile']    = '%s_mcmc.txt'             % prefix_base
            get_BV_para_dict['outfile']     = '%s_out.txt'              % prefix_base
            get_BV_para_dict['seqtype']     = '2'
            get_BV_para_dict['usedata']     = '3'
            get_BV_para_dict['clock']       = '3'
            prep_mcmctree_ctl(get_BV_para_dict, pwd_get_BV_mcmctree_ctl)

            with open(get_BV_js, 'w') as get_BV_js_handle:
                get_BV_js_handle.write('#!/bin/bash\n#SBATCH --ntasks 1\n#SBATCH --cpus-per-task %s\n\n' % js_cpu_num)
                get_BV_js_handle.write('cd %s/%s\n' % (os.getcwd(), get_BV_wd))
                get_BV_js_handle.write('mcmctree %s\n' % get_BV_mcmctree_ctl)

            # prepare files for dating
            para_dod = get_parameter_combinations(para_to_test_dict)
            for para_combination in para_dod:
                mcmctree_ctl         = '%s_%s_mcmctree.ctl'             % (prefix_base, para_combination)
                current_dating_wd    = '%s/%s_DeltaLL_%s_%s_dating_wd'  % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct, para_combination)
                pwd_mcmctree_ctl     = '%s/%s_%s_mcmctree.ctl'          % (current_dating_wd, prefix_base, para_combination)
                js_mcmctree          = '%s/js_%s_DeltaLL_%s_%s.sh'      % (op_dir, deltall_stdout_basename.split('_DeltaLL_stdout')[0], each_keep_pct, para_combination)
                pwd_aln_in_dating_wd = '%s/%s'                          % (current_dating_wd, aln_concatenated)

                # create dating wd and copy tree and alignment files into it
                os.mkdir(current_dating_wd)
                fa2phy(pwd_aln_concatenated_in_aod_wd_fasta, pwd_aln_in_dating_wd)  # sequence in phylip format need to be in one line
                os.system('cp %s %s/' % (pwd_c60_tree_file_rooted_with_time_final, current_dating_wd))

                current_para_dict = para_dod[para_combination]
                current_para_dict['seqfile']  = aln_concatenated
                current_para_dict['treefile'] = c60_tree_file_rooted_with_time_final
                current_para_dict['mcmcfile'] = '%s_%s_mcmc.txt' % (prefix_base, para_combination)
                current_para_dict['outfile']  = '%s_%s_out.txt'  % (prefix_base, para_combination)
                current_para_dict['seqtype']  = '2'
                current_para_dict['usedata']  = '2'

                prep_mcmctree_ctl(current_para_dict, pwd_mcmctree_ctl)

                with open(js_mcmctree, 'w') as js_mcmctree_handle:
                    js_mcmctree_handle.write('#!/bin/bash\n\n')
                    js_mcmctree_handle.write('cd %s/%s\n' % (os.getcwd(), current_dating_wd))
                    js_mcmctree_handle.write('cp ../%s_get_BV_wd/out.BV in.BV\n' % prefix_base)
                    js_mcmctree_handle.write('mcmctree %s\n' % mcmctree_ctl)
                print('Job script for performing dating exported to %s' % js_mcmctree)

            if submit_job is True:
                submit_js(get_BV_js)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-deltall', required=True,                          help='DeltaLL stdout')
    parser.add_argument('-aod',     required=True,                          help='AssessMarkerDeltaLL output dir')
    parser.add_argument('-og',      required=True,                          help='outgroup leaves, one id per line')
    parser.add_argument('-eu',      required=True,                          help='EU tree with time constraints')
    parser.add_argument('-o',       required=True,                          help='dating wd')
    parser.add_argument('-c',       required=False, default='25-50-75-100', help='cutoffs, default: 25-50-75-100')
    parser.add_argument('-mmn',     required=False, default=20, type=int,   help='minimal marker number, default: 20')
    parser.add_argument('-ra',      required=False, default=45, type=int,   help='root age, default: 45')
    parser.add_argument('-qsub',    required=False, action="store_true",    help='submit job scripts for getting in.BV')
    parser.add_argument('-f',       required=False, action="store_true",    help='force overwrite')
    parser.add_argument('-to_test', required=True,                          help='Settings to test')
    args = vars(parser.parse_args())
    dating_ss(args)


'''

cd /Users/songweizhi/Desktop/dating_test
python3 /Users/songweizhi/PycharmProjects/TreeSAK/TreeSAK/dating_ss.py -deltall Marker_set_2_Betts_2018_29_arCOG_Marker2Tree_e30/s10_assess_marker_deltaLL/PA_75_DeltaLL_stdout.txt -aod Marker_set_2_Betts_2018_29_arCOG_Marker2Tree_e30/s11_marker_sets_by_DeltaLL -og out_group.txt -eu 27.nwk -o Marker_set_2_Betts_2018_29_arCOG_Marker2Tree_e30/s12_dating_wd -c 25-50-75-100 -mmn 20 -f 

cd /home-user/wzsong/DateArTree
python3 dating_ss.py -deltall Marker_set_2_Betts_2018_29_arCOG_Marker2Tree_e30/s10_assess_marker_deltaLL/PA_75_DeltaLL_stdout.txt -aod Marker_set_2_Betts_2018_29_arCOG_Marker2Tree_e30/s11_marker_sets_by_DeltaLL -og out_group.txt -eu 27.nwk -o Marker_set_2_Betts_2018_29_arCOG_Marker2Tree_e30/s12_dating_wd -c 25-50-75-100 -mmn 20 -f 

'''



import os
import argparse
import arviz as az
import pandas as pd
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure


CompareMCMC_usage = '''
====================================== CompareMCMC example commands ======================================

TreeSAK CompareMCMC -mx IR_mcmc.txt -my AR_mcmc.txt -lx IR -ly AR -o convergence_plot.png -max 40 -fs 12

cd /Users/songweizhi/Desktop
TreeSAK CompareMCMC -mx /Users/songweizhi/Desktop/Sponge_r220/6_dating/MCMCTree/dating_outputs/topo2p10_clock3_nsample250000_run1_mcmc.txt -my /Users/songweizhi/Desktop/Sponge_r220/6_dating/MCMCTree/dating_outputs/topo2p10_clock3_nsample250000_run2_mcmc.txt -lx IR -ly AR -o convergence_plot.png -max 40 -fs 12

==========================================================================================================
'''


def sep_path_basename_ext(file_in):
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'
    file_basename, file_extension = os.path.splitext(file_name)
    return file_path, file_basename, file_extension


def CompareMCMC():

    # mcmc_txt_x      = args['mx']
    # mcmc_txt_y      = args['my']
    # label_x         = args['lx']
    # label_y         = args['ly']
    # pwd_figure      = args['o']
    # max_axis_value  = args['max']
    # label_fs        = args['fs']

    label_fs = 16
    file_x = '/Users/songweizhi/Desktop/x.txt'
    file_y = '/Users/songweizhi/Desktop/y.txt'
    pwd_figure = '/Users/songweizhi/Desktop/Figures.pdf'
    min_value = 0
    max_value = 1
    max_axis_value = 1
    label_x = []
    num_list_x = []
    x_err_l = []
    x_err_r = []
    line_num_index = 0
    for line in open(file_x):
        line_split = line.strip().split('\t')
        if line_num_index > 0:
            label_x.append(line_split[0])
            num_list_x.append(float(line_split[1]))
            x_err_l.append(float(line_split[1]) - float(line_split[2]))
            x_err_r.append(float(line_split[3]) - float(line_split[1]))
        line_num_index += 1


    label_y = []
    num_list_y = []
    y_err_l = []
    y_err_u = []
    line_num_index = 0
    for line in open(file_y):
        line_split = line.strip().split('\t')
        if line_num_index > 0:
            label_y.append(line_split[0])
            num_list_y.append(float(line_split[1]))
            y_err_l.append(float(line_split[1]) - float(line_split[2]))
            y_err_u.append(float(line_split[3]) - float(line_split[1]))
        line_num_index += 1






    figure(figsize=(6, 6), dpi=300)
    plt.plot([min_value, max_value], [min_value, max_value], color='black', linestyle='dashed', linewidth=1, alpha=0.5)
    plt.scatter(num_list_x, num_list_y, s=0)
    plt.errorbar(num_list_x, num_list_y, xerr=[x_err_l, x_err_r], yerr=[y_err_l, y_err_u],
                 ls='none', ecolor='skyblue', elinewidth=1, alpha=0.5)

    if max_axis_value is not None:
        plt.xlim([0, max_axis_value])
        plt.ylim([0, max_axis_value])

    # Set the font size of xticks and yticks
    plt.xticks(fontsize=label_fs)
    plt.yticks(fontsize=label_fs)
    plt.xlabel(label_x, fontsize=label_fs)
    plt.ylabel(label_y, fontsize=label_fs)

    # write out
    plt.tight_layout()
    plt.savefig(pwd_figure)
    plt.close()

    print('Plot exported to %s, done!' % pwd_figure)


CompareMCMC()

# if __name__ == '__main__':
#
#     # initialize the options parser
#     parser = argparse.ArgumentParser()
#     parser.add_argument('-mx',      required=True,                          help='mcmc.txt for x axis')
#     parser.add_argument('-my',      required=True,                          help='mcmc.txt for y axis')
#     parser.add_argument('-lx',      required=False, default=None,           help='label for x axis')
#     parser.add_argument('-ly',      required=False, default=None,           help='label for y axis')
#     parser.add_argument('-max',     required=False, default=None, type=int, help='maximum axis value')
#     parser.add_argument('-fs',      required=False, default=16, type=int,   help='label font size, default: 16')
#     parser.add_argument('-o',       required=True,                          help='output plot')
#     args = vars(parser.parse_args())
#     CompareMCMC(args)



from __future__ import print_function
import os
import argparse
from Bio import SeqIO


SplitScore1OMA_usage = '''
======================== SplitScore1OMA example commands ========================

# SplitScore1
TreeSAK SplitScore1OMA -i OrthologousGroups.txt -s OrthologousGroupsFasta -o step1_op_dir -t 6 -f
TreeSAK SplitScore1OMA -i OrthologousGroups.txt -s OrthologousGroupsFasta -o step1_op_dir -t 6 -f -u interested_gnm.txt
# Please ensure that all the commands in iqtree_cmds.txt have been executed before proceeding to step 2.

=================================================================================
'''


def select_seq(seq_file, seq_id_list, output_file):
    output_file_handle = open(output_file, 'w')
    for seq_record in SeqIO.parse(seq_file, 'fasta'):
        seq_id = seq_record.id
        if seq_id in seq_id_list:
            output_file_handle.write('>%s\n' % seq_id)
            output_file_handle.write('%s\n' % str(seq_record.seq))
    output_file_handle.close()


def get_gene_tree(oma_op_txt, oma_op_fasta, interested_gnm_txt, cov_cutoff, oma_op_fasta_qualified, iqtree_model, num_of_js_threads, force_overwrite, get_gene_tree_cmd_txt):

    # get the total number of genome
    genome_id_set = set()
    for each_group in open(oma_op_txt):
        if not each_group.startswith('#'):
            for each_gene in each_group.strip().split('\t')[1:]:
                gnm_id = '_'.join(each_gene.split(':')[1].split(' ')[0].split('_')[:-1])
                genome_id_set.add(gnm_id)

    interested_gnm_set = set()
    if interested_gnm_txt is not None:
        for each_gnm in open(interested_gnm_txt):
            interested_gnm_set.add(each_gnm.strip())
    else:
        interested_gnm_set = genome_id_set

    # create output folder
    if os.path.isdir(oma_op_fasta_qualified) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % oma_op_fasta_qualified)
        else:
            print('%s already exist, program exited!' % oma_op_fasta_qualified)
            exit()
    os.system('mkdir %s' % oma_op_fasta_qualified)

    # filter OMA output
    qualified_grp_to_gene_dict = dict()
    for each_group in open(oma_op_txt):
        if not each_group.startswith('#'):
            each_group_split  = each_group.strip().split('\t')
            group_id          = each_group_split[0]
            gene_list_by_gnm  = each_group_split[1:]
            current_gene_list = [i.split(':')[1].split(' ')[0] for i in gene_list_by_gnm]
            current_gnm_list_interested = []
            current_gene_list_interested = []
            for gene in current_gene_list:
                gnm = '_'.join(gene.split('_')[:-1])
                if gnm in interested_gnm_set:
                    current_gnm_list_interested.append(gnm)
                    current_gene_list_interested.append(gene)

            current_cov = len(current_gnm_list_interested) * 100 / len(interested_gnm_set)
            if current_cov >= cov_cutoff:
                qualified_grp_to_gene_dict[group_id] = current_gene_list_interested

    print('The number of orthologous groups with coverage >= %s is %s.' % (cov_cutoff, len(qualified_grp_to_gene_dict)))

    # prepare commands for getting gene tree
    get_gene_tree_cmd_txt_handle = open(get_gene_tree_cmd_txt, 'w')
    for qualified_grp in sorted(list(qualified_grp_to_gene_dict.keys())):
        group_id_only_num = qualified_grp.replace('OMA', '')
        while group_id_only_num[0] == '0':
            group_id_only_num = group_id_only_num[1:]

        # define file name
        og_id               = 'OG%s'                % group_id_only_num
        pwd_seq_file_in     = '%s/%s.fa'            % (oma_op_fasta, og_id)
        pwd_og_seq          = '%s/%s.fa'            % (oma_op_fasta_qualified, og_id)
        pwd_og_aln          = '%s/%s.aln'           % (oma_op_fasta_qualified, og_id)
        pwd_og_aln_trimmed  = '%s/%s_trimmed.aln'   % (oma_op_fasta_qualified, og_id)

        # get sequence
        if len(interested_gnm_set) == len(genome_id_set):
            cp_cmd = 'cp %s %s' % (pwd_seq_file_in, pwd_og_seq)
            os.system(cp_cmd)
        else:
            select_seq(pwd_seq_file_in, qualified_grp_to_gene_dict[qualified_grp], pwd_og_seq)

        # align, trim and iqtree
        mafft_cmd     = 'mafft-einsi --thread %s --quiet %s > %s'                                      % (num_of_js_threads, pwd_og_seq, pwd_og_aln)
        trimal_cmd    = 'trimal -in %s -out %s -automated1'                                            % (pwd_og_aln, pwd_og_aln_trimmed)
        iqtree_cmd    = 'iqtree2 -s %s --seqtype AA -m %s -T %s -B 1000 --quiet --wbtl --prefix %s/%s' % (pwd_og_aln_trimmed, iqtree_model, num_of_js_threads, oma_op_fasta_qualified, og_id)
        cmds_one_line = '%s; %s; %s'                                                                   % (mafft_cmd, trimal_cmd, iqtree_cmd)
        get_gene_tree_cmd_txt_handle.write(cmds_one_line.replace((oma_op_fasta_qualified + '/'), '') + '\n')
    get_gene_tree_cmd_txt_handle.close()


def SplitScore1OMA(args):

    oma_op_txt              = args['i']
    oma_op_fasta            = args['s']
    interested_gnm_txt      = args['u']
    iqtree_model            = args['m']
    cov_cutoff              = args['c']
    force_overwrite         = args['f']
    num_of_js_threads       = args['jst']
    step_1_op_dir           = args['o']

    # define file name
    qualified_og_dir    = '%s/qualified_OGs'    % step_1_op_dir
    iqtree_cmds_txt     = '%s/iqtree_cmds.txt'  % step_1_op_dir

    # create output folder
    if os.path.isdir(step_1_op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % step_1_op_dir)
        else:
            print('%s exist, program exited!' % step_1_op_dir)
            exit()
    os.mkdir(step_1_op_dir)
    os.mkdir(qualified_og_dir)

    # get get_gene_tree
    get_gene_tree(oma_op_txt, oma_op_fasta, interested_gnm_txt, cov_cutoff, qualified_og_dir, iqtree_model, num_of_js_threads, force_overwrite, iqtree_cmds_txt)


if __name__ == '__main__':

    SplitScore1OMA_parser = argparse.ArgumentParser()
    SplitScore1OMA_parser.add_argument('-i',   required=True,                        help='OrthologousGroups.txt, produced by OMA')
    SplitScore1OMA_parser.add_argument('-s',   required=True,                        help='OrthologousGroupsFasta, produced by OMA')
    SplitScore1OMA_parser.add_argument('-u',   required=False, default= None,        help='ID of interested genomes, no file extension')
    SplitScore1OMA_parser.add_argument('-o',   required=True,                        help='output directory')
    SplitScore1OMA_parser.add_argument('-m',   required=False, default='LG+G+I',     help='iqtree_model, default: LG+G+I')
    SplitScore1OMA_parser.add_argument('-c',   required=False, type=int, default=80, help='coverage cutoff, default: 80')
    SplitScore1OMA_parser.add_argument('-f',   required=False, action="store_true",  help='force overwrite')
    SplitScore1OMA_parser.add_argument('-jst', required=False, type=int, default=1,  help='num of threads for inferring gene tree, default: 1')
    args = vars(SplitScore1OMA_parser.parse_args())
    SplitScore1OMA(args)



import os
import argparse
from ete3 import Tree


gnm_leaves_usage = '''
========== gnm_leaves example commands ==========

TreeSAK gnm_leaves -i input.tree -o output.tree

=================================================
'''


def gnm_leaves(args):

    tree_file_in  = args['i']
    tree_file_out = args['o']
    tree_format   = args['fmt']

    if os.path.isfile(tree_file_in) is False:
        print('Tree file not found, program exited!')
        exit()

    t = Tree(tree_file_in, format=tree_format)

    for leaf in t:
        leaf_name = leaf.name
        leaf_name_new = '_'.join(leaf_name.split('_')[:-1])
        leaf.name = leaf_name_new
    t.write(format=tree_format, outfile=tree_file_out)

    print('Done!')


if __name__ == '__main__':

    gnm_leaves_parser = argparse.ArgumentParser()
    gnm_leaves_parser.add_argument('-i',    required=True,                       help='input tree')
    gnm_leaves_parser.add_argument('-o',    required=True,                       help='output tree')
    gnm_leaves_parser.add_argument('-fmt',  required=False, default=1, type=int, help='tree format, default: 1')
    args = vars(gnm_leaves_parser.parse_args())
    gnm_leaves(args)



import argparse
from ete3 import Tree


subset_usage = '''
=================== subset example commands ===================

TreeSAK subset -fmt 1 -i in.tree -k leaves.txt -o subset.tree
TreeSAK subset -fmt 1 -i in.tree -r leaves.txt -o subset.tree

===============================================================
'''


def subset(args):

    tree_file_in    = args['i']
    tree_file_out   = args['o']
    to_keep_txt     = args['k']
    to_remove_txt   = args['r']
    tree_fmt        = args['fmt']


    genomes_to_keep = []
    if (to_keep_txt is None) and (to_remove_txt is None):
        print('Please specify either -k or -r, program exited!')
        exit()

    elif (to_keep_txt is not None) and (to_remove_txt is not None):
        print('Please do NOT specify -k and -r at the same time, program exited!')
        exit()

    elif (to_keep_txt is not None) and (to_remove_txt is None):
        genomes_to_keep = [i.strip() for i in open(to_keep_txt)]

    elif (to_keep_txt is None) and (to_remove_txt is not None):

        genomes_to_remove = [i.strip() for i in open(to_remove_txt)]

        leaf_list = []
        for leaf in Tree(tree_file_in, quoted_node_names=True, format=tree_fmt):
            leaf_name = leaf.name
            leaf_list.append(leaf_name)

        for each_leaf in leaf_list:
            if each_leaf not in genomes_to_remove:
                genomes_to_keep.append(each_leaf)

        if len(leaf_list) == len(genomes_to_keep):
            print('No leaf to remove, program exited!')
            exit()

    input_tree = Tree(tree_file_in, quoted_node_names=True, format=tree_fmt)
    subset_tree = input_tree.copy()
    subset_tree.prune(genomes_to_keep, preserve_branch_length=True)
    subset_tree.write(outfile=tree_file_out)

    print('Subset tree exported to: %s' % tree_file_out)

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-i',      required=True,                       help='input tree file')
    parser.add_argument('-o',      required=True,                       help='output tree file')
    parser.add_argument('-k',      required=False, default=None,        help='leaves to keep')
    parser.add_argument('-r',      required=False, default=None,        help='leaves to remove')
    parser.add_argument('-fmt',    required=False, default=1, type=int, help='tree format, default: 1')
    args = vars(parser.parse_args())
    subset(args)



from __future__ import print_function
import os
import re
import sys
import numpy
import argparse
from ete3 import Tree
import multiprocessing as mp
from operator import itemgetter
from collections import defaultdict


def get_rename_dict(tree_str_in, mag_rename_dict, mag_cluster_dict, sponge_mag_tax_dict, gtdb_gnm_tax_dict):

    # rename dict: {'old_name':'new_name'}

    leaf_rename_dict = {}
    for leaf in Tree(tree_str_in, format=1):

        leaf_name_gnm = '_'.join(leaf.name.split('_')[:-1])
        leaf_name_gnm = mag_rename_dict.get(leaf_name_gnm, leaf_name_gnm)
        leaf_cluster = mag_cluster_dict.get(leaf_name_gnm, 'cluster_0')

        leaf_name_gnm_no_source = leaf_name_gnm
        if '.gtdb' in leaf_name_gnm_no_source:
            leaf_name_gnm_no_source = leaf_name_gnm[:-5]
        if '.ncbi' in leaf_name_gnm:
            leaf_name_gnm_no_source = leaf_name_gnm[:-5]

        # get mag_taxon_str
        gnm_taxon_str = 'NA'
        if leaf_name_gnm_no_source in sponge_mag_tax_dict:
            gnm_taxon_str = sponge_mag_tax_dict[leaf_name_gnm_no_source]
        if leaf_name_gnm_no_source in gtdb_gnm_tax_dict:
            gnm_taxon_str = gtdb_gnm_tax_dict[leaf_name_gnm_no_source]

        # get mag_taxon_str (GCA GCF things)
        if gnm_taxon_str == 'NA':
            mag_id_no_ext_no_source_GCF = leaf_name_gnm_no_source.replace('GCA', 'GCF')
            if mag_id_no_ext_no_source_GCF in gtdb_gnm_tax_dict:
                gnm_taxon_str = gtdb_gnm_tax_dict[mag_id_no_ext_no_source_GCF]

        gnm_taxon_str_no_space = gnm_taxon_str.replace(' ', '_')
        gnm_taxon_str_no_space = gnm_taxon_str_no_space.replace(';', '|')
        leaf_name_new = '%s|%s|strain__%s' % (leaf_cluster, gnm_taxon_str_no_space, '_'.join(leaf.name.split('_')[:-1]))

        leaf_rename_dict[leaf.name] = leaf_name_new

    return leaf_rename_dict


def rename_tree(tree_str_in, rename_dict):

    t_in = Tree(tree_str_in, format=1)
    for leaf in t_in:
        leaf_name = leaf.name
        leaf_name_new = rename_dict.get(leaf_name, leaf_name)
        leaf.name = leaf_name_new

    return t_in.write()


def gtdb_gnm_metadata_parser(gtdb_genome_metadata):

    genome_to_taxon_dict = {}
    genome_to_completeness_dict = {}
    genome_to_contamination_dict = {}
    genome_to_biosample_dict = {}

    col_index = {}
    for each_ref in open(gtdb_genome_metadata):
        each_ref_split = each_ref.strip().split('\t')
        if each_ref.startswith('accession'):
            col_index = {key: i for i, key in enumerate(each_ref_split)}
        else:
            ref_accession = each_ref_split[0][3:]
            gnm_completeness = float(each_ref_split[2])
            gnm_contamination = float(each_ref_split[3])
            gtdb_taxon = each_ref_split[col_index['gtdb_taxonomy']]
            ncbi_biosample = each_ref_split[col_index['ncbi_biosample']]
            genome_to_taxon_dict[ref_accession] = gtdb_taxon
            genome_to_completeness_dict[ref_accession] = gnm_completeness
            genome_to_contamination_dict[ref_accession] = gnm_contamination
            genome_to_biosample_dict[ref_accession] = ncbi_biosample

    return genome_to_completeness_dict, genome_to_contamination_dict, genome_to_taxon_dict, genome_to_biosample_dict


def parse_taxonomy(taxon_name):  # given a taxon name, try to return whatever taxonomic info is available as a list starting with the highest level classification and going lower (or a map?)
    #name_elements = re.split("\|", taxon_name)
    name_elements = taxon_name.split('|')
    #print('name_elements')
    #print(name_elements)

    if (len(name_elements) < 8) or (len(name_elements) > 9):
        print("Nonstandard!")
        quit()

    name_map = {}
    name_map['cluster'] = name_elements[0]
    name_map['domain'] = name_elements[1]
    name_map['phylum'] = name_elements[2]
    name_map['class'] = name_elements[3]
    name_map['order'] = name_elements[4]
    name_map['family'] = name_elements[5]
    name_map['genus'] = name_elements[6]
    name_map['species'] = name_elements[7]
    if len(name_elements) == 9:
        name_map['ncbi_id'] = name_elements[8]
    return name_map


def summarize_taxonomy(name_list, tax_level, name_to_tax_dict):  # take a list of names from a clade and summarize taxonomic info (labels and their frequencies)
    total_size = len(name_list)  # it perhaps makes sense to normalize by the size of the clade
    breakdown = {}
    for name in name_list:
        info = name_to_tax_dict[name]
        if info[tax_level] in breakdown:
            breakdown[info[tax_level]] += 1.0 / float(total_size)
        else:
            breakdown[info[tax_level]] = 1.0 / float(total_size)
    return breakdown


def count_sister_taxa(target_label, tree_in_ml, tree_in_bs, output_file):

    # read in argument
    # target_label = args['l']
    # tree_in_ml   = args['ml']
    # tree_in_bs   = args['bs']
    # output_file  = args['out']

    # edit target_label to make the comparisons at a desired taxonomic level

    # compute the most frequent sister group of each (monophyletic?) group on the tree, to identify trends in gene transfers, "unstable" taxa, etc.

    # read the ML tree, set up the taxonomy stuff, and calculate the number of clades per label, and the sizes of those clades (to report at the end)
    labels = {}
    name_to_tax_info = defaultdict(dict)
    all_tree_leaf_names = []
    ml_tree = Tree(tree_in_ml)  # note that ete3 treats this input tree as rooted
    for leaf in ml_tree:
        taxonomy = parse_taxonomy(leaf.name)
        name_to_tax_info[leaf.name] = taxonomy
        all_tree_leaf_names.append(leaf.name)
        leaf.add_feature("tax", taxonomy[target_label])
        labels[taxonomy[target_label]] = 1
    groups = labels.keys()

    # compute the number of clades (weizhi: monophyletic group) per label in the ML tree, and their sizes
    ML_groups = defaultdict(list)  # the list is the size of each clade, len(list) is the number of clades for that label in the ML tree
    # ML_groups: the number of leaves in each monophyletic groups of the corresponding target_label (e.g. genus)
    for label in groups:
        node_num = 0
        for monophyletic_clade in ml_tree.get_monophyletic(values=[label], target_attr="tax"):  # get monophyletic groups for each target_label (e.g. genus)
            # print('node')
            # print(node)
            size_clade = 0  # get the number of leaf (size_clade) in the monophyletic group
            for leaf in monophyletic_clade:
                size_clade += 1
            ML_groups[label].append(size_clade)
            node_num += 1
        # print('The number of monophyletic clade (node_num) of %s (label):\t%s' % (label, node_num))

    # print()
    # print('Dict holds the number of monophyletic clades per taxon, and their sizes')
    # print('ML_groups:\t %s' % ML_groups)
    # print()

    summary = defaultdict(dict)
    clades_per_group = defaultdict(list)
    treeNum = -1
    for line in open(tree_in_bs):  # read in each bootstrap tree

        treeNum += 1
        tree = Tree(line.rstrip())
        for leaf in tree:
            tax = name_to_tax_info[leaf.name]  # this should set up taxonomy correctly...
            leaf.add_feature("tax", tax[target_label])  # this adds a feature called tax to the leaf, with the attribute of the phylum name
        for label in groups:
            clades_per_group[label].append(0.0)  # setup the clade counting for this particular tree
        tree.unroot()  # Weizhi: why is this

        # iterate over groups that are monophyletic for the taxon label of choice.
        # Choose the smallest sister branch for the comparison. (Assume root is within the larger sister clade (Weizhi:why?))
        for label in groups:

            # print('tree.get_monophyletic(values=[label], target_attr="tax")')
            # print(tree.get_monophyletic(values=[label], target_attr="tax"))
            # print('---------------------------------------------------------------------------------------------------v')
            # print('label: %s' % label)
            monophyletic_clade_index = 1
            for monophyletic_clade in tree.get_monophyletic(values=[label], target_attr="tax"):  # node: monophyletic clade
                clades_per_group[label][treeNum] += 1.0
                # print node.get_ascii()
                sister_clades = monophyletic_clade.get_sisters()

                # print('--------------------v')
                # print('monophyletic clade %s in %s (label)' % (monophyletic_clade_index, label))
                monophyletic_clade_index += 1
                #print(monophyletic_clade)
                # print(len(sisters))
                # for leaf in sisters[0]:
                #     print(leaf.name)
                # print(sisters)
                # print('sisters of current monophyletic clade')
                sister_index = 1
                for each_sister in sister_clades:
                    current_sister_leaf_list = []
                    for leaf in each_sister:
                        current_sister_leaf_list.append(leaf.name)
                    # print('sister %s has %s leaves: %s' % (sister_index, len(current_sister_leaf_list), ','.join([])))
                    sister_index += 1

                if monophyletic_clade.is_root():  # monophyletic clade is root
                    continue

                # Weizhi: bifurcation
                elif len(sister_clades) == 1:  # not at the trifurcation. Do something a bit hacky to find the bigger sister clade

                    taxa_in_sister = []
                    for leaf in sister_clades[0]:
                        taxa_in_sister.append(leaf.name)

                    size_sister = len(taxa_in_sister)

                    taxa_in_group = []
                    for leaf in monophyletic_clade:
                        taxa_in_group.append(leaf.name)

                    taxa_in_other_groups = []  # what does OG mean? (other groups?)
                    for leaf_name in all_tree_leaf_names:
                        if leaf_name in taxa_in_sister:
                            continue
                        elif leaf_name in taxa_in_group:
                            continue
                        else:
                            taxa_in_other_groups.append(leaf_name)
                    size_other_groups = len(taxa_in_other_groups)

                    sister_tax = {}  # taxa in the smaller groups (either the sister group or the OG)
                    if size_other_groups > size_sister:
                        sister_tax = summarize_taxonomy(taxa_in_sister, target_label, name_to_tax_info)
                    else:
                        sister_tax = summarize_taxonomy(taxa_in_other_groups, target_label, name_to_tax_info)

                    # print('size_sister: %s' % size_sister)
                    # print('size_other_groups: %s' % size_other_groups)
                    # print('sister_tax (not really, actually taxa in the smaller one (either the sister group or the OG))')
                    # print(sister_tax)

                    # store the tax info of the sister group
                    for element in sister_tax:
                        # print('element: %s' % element)
                        #print('summary[label]: %s' % summary[label])
                        if element in summary[label]:
                            summary[label][element] += sister_tax[element]
                            #print('summary (in): %s' % summary)
                        else:
                            summary[label][element] = sister_tax[element]
                            #print('summary (not in): %s' % summary)

                else:  # trifurcation in tree. Just treat the two sisters in the same way.

                    taxa_in_sisters_1 = []
                    for leaf in sister_clades[0]:
                        taxa_in_sisters_1.append(leaf.name)

                    taxa_in_sisters_2 = []
                    for leaf in sister_clades[1]:
                        taxa_in_sisters_2.append(leaf.name)

                    # get the size of two sisters
                    size_s1 = len(taxa_in_sisters_1)
                    size_s2 = len(taxa_in_sisters_2)

                    # print('size_s1: %s' % size_s1)
                    # print('size_s2: %s' % size_s2)

                    # get taxa in the smaller sister group
                    sister_tax = {}
                    if size_s1 > size_s2:
                        sister_tax = summarize_taxonomy(taxa_in_sisters_2, target_label, name_to_tax_info)
                    else:
                        sister_tax = summarize_taxonomy(taxa_in_sisters_1, target_label, name_to_tax_info)

                    # print('sister_tax (taxa in the smaller sister group)')
                    # print(sister_tax)

                    for element in sister_tax:
                        if element in summary[label]:
                            summary[label][element] += sister_tax[element]
                        else:
                            summary[label][element] = sister_tax[element]

            #     print('--------------------^')
            #     print()
            # print('---------------------------------------------------------------------------------------------------^')

    # now print out some kind of summary. For each label, the sorted list of sister taxa and their frequencies?
    outh = open(output_file, "w")
    for label in summary:
        num_groups = len(ML_groups[label])
        size_str = ''
        if num_groups == 1:
            size_str = ML_groups[label][0]
        else:
            size_str = ','.join(str(x) for x in (sorted(ML_groups[label], reverse=True)))

        avg_num_clades = float("{0:.4f}".format(numpy.mean(clades_per_group[label])))
        total_num_clades = numpy.sum(clades_per_group[label])
        sorted_sisters = sorted(summary[label].items(), key=itemgetter(1), reverse=True)

        # if label == 'g__TA-20':
        #     print('ML_groups[label]:\t%s'               % ML_groups[label])
        #     print('clades_per_group[label]:\t%s'        % clades_per_group[label])
        #     print('avg_num_clades (mean of list):\t%s'  % numpy.mean(clades_per_group[label]))
        #     print('total_num_clades (sum of list):\t%s' % total_num_clades)
        #     print('summary[label]:\t%s'                 % summary[label])
        #     print('sorted_sisters\t%s'                  % sorted_sisters)

        for tup in sorted_sisters:
            double_normalize = float(tup[1]) / float(total_num_clades)  # normalize the frequencies by the total number of clades, to account for different bootstrap numbers/MCMC sample numbers
            double_normalize = float("{0:.4f}".format(double_normalize))

            str_to_write = '%s\t%s\t%s\t%s\t%s\t%s' % (label, tup[0], float("{0:.4f}".format(tup[1])), avg_num_clades, double_normalize, size_str)
            outh.write(str_to_write + '\n')
            # if label == 'g__TA-20':
            #     print(str_to_write)
    outh.close()


def count_sister_taxa_worker(arg_list):

    mag_rename_dict                 = arg_list[0]
    mag_cluster_dict                = arg_list[1]
    sponge_archaeal_MAG_tax_dict    = arg_list[2]
    gtdb_ar_gnm_tax_dict            = arg_list[3]
    tree_ml                         = arg_list[4]
    ufboot_file                     = arg_list[5]
    target_label                    = arg_list[6]
    tree_ml_renamed                 = arg_list[7]
    ufboot_file_renamed             = arg_list[8]
    count_sister_taxa_op_txt        = arg_list[9]
    gene_id                         = arg_list[10]
    renamed_gnm_to_cluster_dir      = arg_list[11]

    # rename ml tree
    tree_ml_renamed_handle = open(tree_ml_renamed, 'w')
    current_tree_rename_dict = get_rename_dict(tree_ml, mag_rename_dict, mag_cluster_dict, sponge_archaeal_MAG_tax_dict, gtdb_ar_gnm_tax_dict)
    tree_ml_str_renamed = rename_tree(tree_ml, current_tree_rename_dict)
    tree_ml_renamed_handle.write(tree_ml_str_renamed + '\n')
    tree_ml_renamed_handle.close()

    current_renamed_gnm_to_cluster_txt = '%s/%s.txt' % (renamed_gnm_to_cluster_dir, gene_id)
    current_renamed_gnm_to_cluster_txt_handle = open(current_renamed_gnm_to_cluster_txt, 'w')
    for each_leaf in current_tree_rename_dict:
        renamed_leaf = current_tree_rename_dict[each_leaf]
        cluster_id = renamed_leaf.split('|')[0]
        current_renamed_gnm_to_cluster_txt_handle.write('%s\t%s\n' % (renamed_leaf, cluster_id))
    current_renamed_gnm_to_cluster_txt_handle.close()

    # rename ufboot trees
    ufboot_file_renamed_handle = open(ufboot_file_renamed, 'w')
    for each_tree in open(ufboot_file):
        tree_str = each_tree.strip()
        current_tree_rename_dict = get_rename_dict(tree_str, mag_rename_dict, mag_cluster_dict, sponge_archaeal_MAG_tax_dict, gtdb_ar_gnm_tax_dict)
        tree_str_renamed = rename_tree(tree_str, current_tree_rename_dict)
        ufboot_file_renamed_handle.write(tree_str_renamed + '\n')
    ufboot_file_renamed_handle.close()

    # count_sister_taxa
    count_sister_taxa(target_label, tree_ml_renamed, ufboot_file_renamed, count_sister_taxa_op_txt)


def run_count_sister_taxa(genome_metadata_ar53_r207_Mac, sponge_MAG_GTDB_archaea, hog_id_txt, contree_and_ufboot_dir, archaeal_mags_renamed_for_prokka_txt, gnm_cluster_txt, target_label, num_threads, output_dir):

    # define file name
    renamed_gnm_to_cluster_dir              = '%s/renamed_genome_to_cluster'            % output_dir
    renamed_gnm_to_cluster_tmp_txt          = '%s/renamed_genome_to_cluster_tmp.txt'    % output_dir
    renamed_gnm_to_cluster_txt              = '%s/renamed_genome_to_cluster.txt'        % output_dir
    renamed_gnm_to_cluster_iTOL_txt         = '%s/renamed_genome_to_cluster_iTOL.txt'   % output_dir
    renamed_contree_dir                     = '%s/renamed_contree'                      % output_dir
    renamed_ufboot_dir                      = '%s/renamed_ufboot'                       % output_dir
    count_sister_taxa_op_dir                = '%s/count_sister_taxa_op'                 % output_dir

    os.mkdir(output_dir)
    os.mkdir(renamed_contree_dir)
    os.mkdir(renamed_ufboot_dir)
    os.mkdir(count_sister_taxa_op_dir)
    os.mkdir(renamed_gnm_to_cluster_dir)

    _, _, gtdb_ar_gnm_tax_dict, _ = gtdb_gnm_metadata_parser(genome_metadata_ar53_r207_Mac)

    sponge_archaeal_MAG_tax_dict = {}
    for each in open(sponge_MAG_GTDB_archaea):
        if not each.startswith('user_genome'):
            each_split = each.strip().split('\t')
            sponge_archaeal_MAG_tax_dict[each_split[0]] = each_split[1]

    hog_list = []
    for each_hog in open(hog_id_txt):
        hog_list.append(each_hog.strip())

    mag_cluster_dict = {}
    for each_gnm in open(gnm_cluster_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        mag_cluster_dict[each_gnm_split[1]] = each_gnm_split[0]

    mag_rename_dict = {}
    for each_mag in open(archaeal_mags_renamed_for_prokka_txt):
        each_mag_split = each_mag.strip().split('\t')
        before_rename = each_mag_split[0]
        after_rename = each_mag_split[1]
        mag_rename_dict[after_rename] = before_rename

    argument_lol = []
    for og_id in hog_list:

        # define file name
        tree_ml                  = '%s/%s_iqtree.contree'               % (contree_and_ufboot_dir, og_id)
        ufboot_file              = '%s/%s_iqtree.ufboot'                % (contree_and_ufboot_dir, og_id)
        tree_ml_renamed          = '%s/%s_iqtree_renamed.contree'       % (renamed_contree_dir, og_id)
        ufboot_file_renamed      = '%s/%s_iqtree_renamed.ufboot'        % (renamed_ufboot_dir, og_id)
        count_sister_taxa_op_txt = '%s/%s_iqtree_count_sister_taxa.txt' % (count_sister_taxa_op_dir, og_id)

        current_arg_list = [mag_rename_dict, mag_cluster_dict, sponge_archaeal_MAG_tax_dict, gtdb_ar_gnm_tax_dict, tree_ml, ufboot_file, target_label, tree_ml_renamed, ufboot_file_renamed, count_sister_taxa_op_txt, og_id, renamed_gnm_to_cluster_dir]
        argument_lol.append(current_arg_list)

    # run with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(count_sister_taxa_worker, argument_lol)
    pool.close()
    pool.join()

    # combine renamed_gnm_to_cluster files
    os.system('cat %s/*.txt > %s' % (renamed_gnm_to_cluster_dir, renamed_gnm_to_cluster_tmp_txt))
    os.system('cat %s | sort | uniq > %s' % (renamed_gnm_to_cluster_tmp_txt, renamed_gnm_to_cluster_txt))
    BioSAK_iTOL_cmd = 'BioSAK iTOL -ColorRange -lg %s -lt Cluster -out %s' % (renamed_gnm_to_cluster_txt, renamed_gnm_to_cluster_iTOL_txt)
    os.system(BioSAK_iTOL_cmd)


####################################################### Test 2023-07-11 ########################################################

genome_metadata_ar53_r207_Mac           = '/Users/songweizhi/DB/GTDB_r207/ar53_metadata_r207.tsv'
sponge_MAG_GTDB_archaea                 = '/Users/songweizhi/Documents/Research/Sponge_Hologenome/0_backup/0_metadata_mac/Sponge_MAGs_1677.ar53.summary.tsv'
hog_id_txt                              = '/Users/songweizhi/Documents/Research/Sponge_Hologenome/0_backup/5_Archaeal_tree_50_5_Markers_by_split_wd/HOG_id.txt'
contree_and_ufboot_dir                  = '/Users/songweizhi/Documents/Research/Sponge_Hologenome/0_backup/5_Archaeal_tree_50_5_Markers_by_split_wd/contree_and_ufboot_files'
archaeal_mags_renamed_for_prokka_txt    = '/Users/songweizhi/Documents/Research/Sponge_Hologenome/0_backup/0_metadata_mac/Archaeal_mags_renamed_for_prokka.txt'
gnm_cluster_txt                         = '/Users/songweizhi/Documents/Research/Sponge_Hologenome/0_backup/5_Archaeal_tree_50_5_Markers_by_split_wd/genome_clusters_v1.txt'
target_label                            = 'cluster'
num_threads                             = 10
output_dir                              = '/Users/songweizhi/Desktop/count_sister_taxa_op_Test_2023_07_11'

run_count_sister_taxa(genome_metadata_ar53_r207_Mac, sponge_MAG_GTDB_archaea, hog_id_txt, contree_and_ufboot_dir, archaeal_mags_renamed_for_prokka_txt, gnm_cluster_txt, target_label, num_threads, output_dir)



import math
import os.path
import random
import argparse
import seaborn as sns


'''
TreeSAK iTOL -ColorLabel -lg Mag_genus.txt -gc genus_color.txt-o ColorLabel_genus.txt
'''

iTOL_usage = '''
==================================== iTOL example commands ====================================

# Example commands
TreeSAK iTOL -Labels -ll new_Mag_name.txt -o Mag_name_iTOL_Labels.txt
TreeSAK iTOL -ColoredLabel -lg leaf_group.txt -gc group_color.txt -o iTOL_ColoredLabel.txt
TreeSAK iTOL -ColoredLabel -lg leaf_group.txt -gc group_color.txt -o iTOL_ColoredLabel.txt -ll new_Mag_name.txt
TreeSAK iTOL -Binary -lm Binary_matrix.txt -lt Enzyme -o Presence_Absence_iTOL.txt -cc "#F1BB83"
TreeSAK iTOL -Binary -lm Binary_matrix.txt -lt Enzyme -o Presence_Absence_iTOL.txt -cc col_color.txt
TreeSAK iTOL -BinaryID -id MagID.txt -lt dRep95 -o dRep95_representatives_iTOL.txt
TreeSAK iTOL -Heatmap -lm MagAbundance.txt -lt Abundance -o Heatmap_abundance.txt
TreeSAK iTOL -SimpleBar -lv MagSize.txt -scale 0-3-6-9 -lt Size -o SimpleBar_size.txt
TreeSAK iTOL -ColorStrip -lg MagTaxon.txt -lt Phylum -gc phylum_color.txt -o ColorStrip_taxon.txt
TreeSAK iTOL -ColorRange -lg MagTaxon.txt -lt Phylum -gc phylum_color.txt -o ColorRange_taxon.txt
TreeSAK iTOL -ColorRange -taxon Taxonomy.txt -rank f -lt Family -o ColorRange_taxon.txt
TreeSAK iTOL -ColorClade -lg Mag_genus.txt -gc genus_color.txt-o ColorClade_genus.txt
TreeSAK iTOL -ExternalShape -lm identity_matrix.txt -lt Identity -scale 25-50-75-100 -o ExternalShape_identity.txt
TreeSAK iTOL -PieChart -lv MagCompleteness.txt -lt Completeness -o PieChart_completeness.txt
TreeSAK iTOL -Collapse -lg MagTaxon.txt -o Collapse_by_taxon.txt

# Leaf-to-Group file format (-lg, tab separated, no header)
genome_1	Bacteria
genome_2	Archaea

# taxonomy file format (-taxon, tab separated, GTDB-format taxononomy string)
genome_1	d__Bacteria;p__Proteobacteria;c__Alphaproteobacteria;o__Dongiales;f__Dongiaceae;g__Dongia;s__Dongia mobilis
genome_2	d__Bacteria;p__Proteobacteria;c__Gammaproteobacteria;o__Arenicellales;f__LS-SOB;g__VYGS01;s__

# Group-to-Color and Column-to-Color file format (-gc and -cc, tab separated, no header)
bac	#CCCC00
ar	#9999FF
# Please note you can only specify one color for Binary data, provide with -gc lightblue or -gc "#85C1E9"

# Leaf-to-Value file format (-lv, tab separated, no header)
genome_1	6.15
genome_2	6.63

# Leaf-to-Matrix file format (-lm, tab separated, header required!!!)
Genome_id SampleA   SampleB   SampleC
genome_1	6.15    2.23    1.56
genome_2	6.63    1.72    2.55

===============================================================================================
'''


def get_color_list(color_num):

    if color_num <= 8:
        color_list_combined = ['#3787c0', '#39399f', '#ffb939', '#399f39', '#9f399f', '#fb694a', '#9f9f39', '#959595']

    elif 8 < color_num <= 16:
        color_list_combined = ['#2b7bba', '#89bedc', '#2e2e99', '#8a8acc', '#ffa500', '#ffc55c', '#2e992e', '#8acc8a', '#992e99', '#cc8acc', '#d52221', '#fc8161', '#99992e', '#cccc8a', '#5c5c5c', '#adadad']

    else:
        color_num_each = math.ceil(color_num/8) + 2
        color_list_1 = sns.color_palette('Blues', n_colors=color_num_each).as_hex()
        color_list_2 = sns.light_palette('navy',   n_colors=color_num_each).as_hex()
        color_list_3 = sns.light_palette('orange', n_colors=color_num_each).as_hex()
        color_list_4 = sns.light_palette('green',  n_colors=color_num_each).as_hex()
        color_list_5 = sns.light_palette('purple', n_colors=color_num_each).as_hex()
        color_list_6 = sns.color_palette('Reds',  n_colors=color_num_each).as_hex()
        color_list_7 = sns.light_palette('olive',  n_colors=color_num_each).as_hex()
        color_list_8 = sns.color_palette('Greys', n_colors=color_num_each).as_hex()

        color_list_combined = []
        for color_list in [color_list_1, color_list_2, color_list_3, color_list_4, color_list_5, color_list_6, color_list_7, color_list_8]:
            for color in color_list[2:][::-1]:
                color_list_combined.append(color)

    color_list_to_return = random.sample(color_list_combined, color_num)

    color_list_to_return_sorted = []
    for color_to_return in color_list_combined:
        if color_to_return in color_list_to_return:
            color_list_to_return_sorted.append(color_to_return)

    random.shuffle(color_list_to_return_sorted)

    return color_list_to_return_sorted


def scale_str_to_size_list(scale_str):

    scale_list = scale_str.split('-')
    scale_list = [float(i) for i in scale_list]

    shape_size_list = []
    if scale_list[0] == 0:
        shape_size_list = [0]
        for each_value in scale_list[1:-1]:
            current_size = each_value/scale_list[-1]
            shape_size_list.append(current_size)
        shape_size_list.append(1)

    if scale_list[0] != 0:
        shape_size_list = [0.1]
        interval_num = len(scale_list) - 1
        interval_value = (1 - 0.1)/interval_num
        n = 1
        for each_value in scale_list[1:-1]:
            shape_size_list.append(interval_value * n + 0.1)
            n += 1
        shape_size_list.append(1)

    return shape_size_list


def iTOL(args):

    # read in arguments
    Labels                  = args['Labels']
    ColoredLabel            = args['ColoredLabel']
    ColorStrip              = args['ColorStrip']
    ColorRange              = args['ColorRange']
    ColorClade              = args['ColorClade']
    ColorLabel              = args['ColorLabel']
    SimpleBar               = args['SimpleBar']
    Heatmap                 = args['Heatmap']
    ExternalShape           = args['ExternalShape']
    Binary                  = args['Binary']
    BinaryID                = args['BinaryID']
    Connection              = args['Connection']
    PieChart                = args['PieChart']
    Collapse                = args['Collapse']
    MultiStyleLabel         = args['MultiStyleLabel']
    leaf_id_txt             = args['id']
    LeafGroup               = args['lg']
    GroupColor              = args['gc']
    ColumnColor_txt         = args['cc']
    LeafValue               = args['lv']
    LeafLabel               = args['ll']
    LeafMatrix              = args['lm']
    d2r                     = args['dr']
    scale_str               = args['scale']
    show_color_strip_legend = args['legend']
    LegendTitle             = args['lt']
    show_strip_labels       = args['show_strip_labels']
    FileOut                 = args['o']
    BinaryShape             = args['BinaryShape']
    BinaryColor             = args['BinaryColor']

    # General
    STRIP_WIDTH                 = 100
    MARGIN                      = 20
    branch_width                = 2

    # SimpleBar
    SimpleBar_COLOR             = 'grey'
    SimpleBar_WIDTH             = 300
    SimpleBar_HEIGHT_FACTOR     = 0.8
    SimpleBar_BORDER_WIDTH      = 0
    SimpleBar_SCALE_COLOR       = '#696969'
    SimpleBar_SCALE_WIDTH       = 1
    SimpleBar_SCALE_DASHED      = 1
    SimpleBar_SCALE_FontSize    = 2

    # Heatmap
    Heatmap_STRIP_WIDTH         = 30

    # check the number of specified file type
    True_num = 0
    for file_type in [Labels, ColorStrip, ColorRange, SimpleBar, Heatmap, ExternalShape, Binary, BinaryID, PieChart, Collapse, ColorClade, ColorLabel, ColoredLabel]:
        if file_type is True:
            True_num += 1

    if True_num == 0:
        print('Please specify one file type, choose from -ColorStrip, -ColorRange, -SimpleBar, -Heatmap, -ExternalShape, -Binary, -BinaryID or -Collapse')
        exit()
    if True_num > 1:
        print('Please specify one file type ONLY, choose from -ColorStrip, -ColorRange, -SimpleBar, -Heatmap, -ExternalShape, -Binary, -BinaryID or -Collapse')
        exit()

    ####################################################################################################################

    if ColoredLabel is True:

        leaf_set = set()
        leaf_label_dict = dict()
        if LeafLabel is not None:
            for each in open(LeafLabel):
                each_split = each.strip().split('\t')
                leaf_label_dict[each_split[0]] = each_split[1]
                leaf_set.add(each_split[0])

        leaf_group_dict = dict()
        for each in open(LeafGroup):
            each_split = each.strip().split('\t')
            leaf_group_dict[each_split[0]] = each_split[1]
            leaf_set.add(each_split[0])

        group_color_dict = dict()
        for each in open(GroupColor):
            each_split = each.strip().split('\t')
            group_color_dict[each_split[0]] = each_split[1]

        FileOut_handle = open(FileOut, 'w')
        FileOut_handle.write('DATASET_TEXT\n')
        FileOut_handle.write('SEPARATOR TAB\n')
        FileOut_handle.write('DATASET_LABEL\t%s\n' % 'ColoredLabel')
        FileOut_handle.write('\nDATA\n')
        for leaf in leaf_set:
            leaf_label = leaf_label_dict.get(leaf, leaf)
            leaf_group = leaf_group_dict.get(leaf, 'na')
            leaf_color = group_color_dict.get(leaf_group, '#000000')
            FileOut_handle.write('%s\t%s\t-1\t%s\tnormal\t1\t0\n' % (leaf, leaf_label, leaf_color))
        FileOut_handle.close()

    ####################################################################################################################

    # Prepare ColorStrip and ColorRange file
    if (ColorStrip is True) or (ColorRange is True):

        Leaf_to_Group_dict = {}
        Group_list = []
        for each_leaf in open(LeafGroup):
            each_leaf_split = each_leaf.strip().split('\t')
            Leaf_to_Group_dict[each_leaf_split[0]] = each_leaf_split[1]
            if each_leaf_split[1] not in Group_list:
                Group_list.append(each_leaf_split[1])

        Group_to_Color_dict = {}
        if GroupColor is None:
            color_list = get_color_list(len(Group_list))
            Group_to_Color_dict = dict(zip(Group_list, color_list))
        else:
            # get groups with provided color
            Group_to_provided_Color_dict = dict()
            group_with_provided_color_list = []
            for each_group in open(GroupColor):
                each_group_split = each_group.strip().split('\t')
                group_id = each_group_split[0]
                color_code = each_group_split[1]
                if group_id in Group_list:
                    Group_to_provided_Color_dict[group_id] = color_code
                    group_with_provided_color_list.append(group_id)

            # assign colors to the rest groups
            group_without_color_list = []
            for each_group in Group_list:
                if each_group not in group_with_provided_color_list:
                    group_without_color_list.append(each_group)
            if len(group_without_color_list) > 0:
                color_list_unprovided = get_color_list(len(group_without_color_list))
                Group_to_Color_dict_unprovided = dict(zip(group_without_color_list, color_list_unprovided))
                for each_group in Group_to_Color_dict_unprovided:
                    Group_to_Color_dict[each_group] = Group_to_Color_dict_unprovided[each_group]

            # combine two dict
            for each_group in Group_to_provided_Color_dict:
                Group_to_Color_dict[each_group] = Group_to_provided_Color_dict[each_group]

        group_list = [i for i in Group_to_Color_dict]
        color_list = [Group_to_Color_dict[i] for i in group_list]

        FileOut_handle = open(FileOut, 'w')

        # write out header
        if ColorStrip is True:
            FileOut_handle.write('DATASET_COLORSTRIP\n')
            FileOut_handle.write('SEPARATOR TAB\n')
            FileOut_handle.write('DATASET_LABEL\t%s\n' % LegendTitle)
            FileOut_handle.write('SHOW_LABELS\t1\n')
            FileOut_handle.write('LABEL_SHIFT\t30\n')
            FileOut_handle.write('LABEL_ROTATION\t45\n')
            FileOut_handle.write('SIZE_FACTOR\t2\n')
            FileOut_handle.write('COLOR_BRANCHES\t0\n')

        if ColorRange is True:
            FileOut_handle.write('TREE_COLORS\n')
            FileOut_handle.write('SEPARATOR TAB\n')
            FileOut_handle.write('DATASET_LABEL\t%s_ColorRange\n' % LegendTitle)

        # write out strip attributes
        if ColorStrip is True:
            FileOut_handle.write('\n# customize strip attributes here\n')
            FileOut_handle.write('STRIP_WIDTH\t%s\n' % STRIP_WIDTH)
            FileOut_handle.write('MARGIN\t%s\n'      % MARGIN)

            if show_color_strip_legend is False:
                FileOut_handle.write('\n# customize labels on the trip\n')
                if show_strip_labels is True:
                    FileOut_handle.write('SHOW_STRIP_LABELS\t1\n')
                else:
                    FileOut_handle.write('SHOW_STRIP_LABELS\t0\n')
                FileOut_handle.write('STRIP_LABEL_POSITION\tcenter\n')
                FileOut_handle.write('STRIP_LABEL_ROTATION\t90\n')
            else:
                FileOut_handle.write('\n# uncomment to show labels on the trip\n')
                if show_strip_labels is True:
                    FileOut_handle.write('# SHOW_STRIP_LABELS\t1\n')
                else:
                    FileOut_handle.write('# SHOW_STRIP_LABELS\t0\n')
                FileOut_handle.write('# STRIP_LABEL_POSITION\tcenter\n')
                FileOut_handle.write('# STRIP_LABEL_ROTATION\t90\n')

        # write out legend info
        if show_color_strip_legend is True:
            FileOut_handle.write('\n# customize legend\n')
            FileOut_handle.write('LEGEND_TITLE\t%s\n' % LegendTitle)
            FileOut_handle.write('LEGEND_SHAPES\t%s\n' % '\t'.join(['1' for i in Group_to_Color_dict]))
            FileOut_handle.write('LEGEND_COLORS\t%s\n' % '\t'.join(color_list))
            FileOut_handle.write('LEGEND_LABELS\t%s\n' % '\t'.join(group_list))
        else:
            FileOut_handle.write('\n# comment this section if you want to hide the legend\n')
            FileOut_handle.write('LEGEND_TITLE\t%s\n' % LegendTitle)
            FileOut_handle.write('LEGEND_SHAPES\t%s\n' % '\t'.join(['1' for i in Group_to_Color_dict]))
            FileOut_handle.write('LEGEND_COLORS\t%s\n' % '\t'.join(color_list))
            FileOut_handle.write('LEGEND_LABELS\t%s\n' % '\t'.join(group_list))

        FileOut_handle.write('\n# provide data here\nDATA\n')
        for leaf in Leaf_to_Group_dict:
            node_group = Leaf_to_Group_dict[leaf]
            leaf_color = Group_to_Color_dict[node_group]

            if ColorStrip is True:
                FileOut_handle.write('%s\t%s\t%s\n' % (leaf, leaf_color, node_group))
            if ColorRange is True:
                FileOut_handle.write('%s\trange\t%s\t%s\n' % (leaf, leaf_color, node_group))
        FileOut_handle.close()

    ####################################################################################################################

    # Prepare SimpleBar file
    if SimpleBar is True:

        if scale_str is None:
            print('Please provide scale values for barchart, in format 0-3-6-9')
            exit()

        # read in leaf value into dict
        leaf_value_dict = {}
        max_value = None
        for leaf_value in open(LeafValue):
            leaf_value_split = leaf_value.strip().split('\t')
            leaf_value_dict[leaf_value_split[0]] = float(leaf_value_split[1])

            # get max value
            if max_value == None:
                max_value = float(leaf_value_split[1])
            else:
                if float(leaf_value_split[1]) > max_value:
                    max_value = float(leaf_value_split[1])

        SimpleBar_FileOut_handle = open(FileOut, 'w')
        SimpleBar_FileOut_handle.write('DATASET_SIMPLEBAR\n')
        SimpleBar_FileOut_handle.write('# Reference: https://itol.embl.de/help/dataset_simplebar_template.txt\n')
        SimpleBar_FileOut_handle.write('\nSEPARATOR TAB\n')
        SimpleBar_FileOut_handle.write('\n# customize barchart attributes here\n')
        SimpleBar_FileOut_handle.write('DATASET_LABEL\t%s\n'    % LegendTitle)
        SimpleBar_FileOut_handle.write('COLOR\t%s\n'            % SimpleBar_COLOR)
        SimpleBar_FileOut_handle.write('WIDTH\t%s\n'            % SimpleBar_WIDTH)
        SimpleBar_FileOut_handle.write('MARGIN\t%s\n'           % MARGIN)
        SimpleBar_FileOut_handle.write('HEIGHT_FACTOR\t%s\n'    % SimpleBar_HEIGHT_FACTOR)
        SimpleBar_FileOut_handle.write('BORDER_WIDTH\t%s\n'     % SimpleBar_BORDER_WIDTH)

        # write out scale attributes
        scale_attributes_list = []
        for scale_value in scale_str.split('-'):
            scale_attributes = '%s-%s-%s-%s-%s-%s' % (scale_value, scale_value, SimpleBar_SCALE_COLOR, SimpleBar_SCALE_WIDTH, SimpleBar_SCALE_DASHED, SimpleBar_SCALE_FontSize)
            scale_attributes_list.append(scale_attributes)

        SimpleBar_FileOut_handle.write('\n# customize scale attributes here\n')
        SimpleBar_FileOut_handle.write('# format: VALUE-LABEL-COLOR-WIDTH-DASHED-LABEL_SCALE_FACTOR, LABEL_SCALE_FACTOR controls font size\n')
        SimpleBar_FileOut_handle.write('DATASET_SCALE\t%s\n' % '\t'.join(scale_attributes_list))
        SimpleBar_FileOut_handle.write('\n# provide data here\n')
        SimpleBar_FileOut_handle.write('DATA\n')

        for leaf in leaf_value_dict:
            SimpleBar_FileOut_handle.write('%s\t%s\n' % (leaf, leaf_value_dict[leaf]))

        SimpleBar_FileOut_handle.close()

    ####################################################################################################################

    # Prepare Binary file
    if Binary is True:

        Column_to_Color_dict = {}
        ColumnColor = 'red'
        if ColumnColor_txt is not None:
            if os.path.isfile(ColumnColor_txt) is True:
                for each_col in open(ColumnColor_txt):
                    each_col_split = each_col.strip().split('\t')
                    Column_to_Color_dict[each_col_split[0]] = each_col_split[1]
            else:
                ColumnColor = ColumnColor_txt

        # get color
        if GroupColor is not None:
            if os.path.isfile(GroupColor) is True:
                print('Only one color can be specified for Binary data, here are two examples:\n-gc lightblue\n-gc "#85C1E9"')
                print('Program exited!')
                exit()
            else:
                Binary_color = GroupColor
        else:
            Binary_color = 'red'

        Binary_FileOut_handle = open(FileOut, 'w')
        line_index = 0
        for each_line in open(LeafMatrix):
            each_line_split = each_line.strip().split('\t')
            if line_index == 0:

                # get header list
                if each_line.startswith('\t'):
                    col_name_list = each_line_split
                else:
                    col_name_list = each_line_split[1:]

                col_color_list = [Column_to_Color_dict.get(i, ColumnColor) for i in col_name_list]

                Binary_FileOut_handle.write('DATASET_BINARY\n\nSEPARATOR TAB\nDATASET_LABEL\t%s\nCOLOR\t%s\n' % (LegendTitle, Binary_color))
                Binary_FileOut_handle.write('SHOW_LABELS\t1\nLABEL_ROTATION\t45\nLABEL_SHIFT\t5\n')
                Binary_FileOut_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(col_name_list))
                Binary_FileOut_handle.write('FIELD_COLORS\t%s\n' % '\t'.join(col_color_list))
                Binary_FileOut_handle.write('MARGIN\t10\n')
                Binary_FileOut_handle.write('HORIZONTAL_GRID\t0\n')
                Binary_FileOut_handle.write('VERTICAL_GRID\t0\n')
                Binary_FileOut_handle.write('FIELD_SHAPES\t%s\n' % '\t'.join(['2'] * len(col_name_list)))
                Binary_FileOut_handle.write('#1: rectangle\n')
                Binary_FileOut_handle.write('#2: circle\n')
                Binary_FileOut_handle.write('#3: star\n')
                Binary_FileOut_handle.write('#4: right pointing triangle\n')
                Binary_FileOut_handle.write('#5: left pointing triangle\n')
                Binary_FileOut_handle.write('#6: check mark\n')
                Binary_FileOut_handle.write('\nDATA\n')
            else:
                Binary_FileOut_handle.write(each_line)
            line_index += 1
        Binary_FileOut_handle.close()

    ####################################################################################################################

    # Prepare BinaryID file
    if BinaryID is True:
        BinaryID_FileOut_handle = open(FileOut, 'w')
        BinaryID_FileOut_handle.write('DATASET_BINARY\n\nSEPARATOR TAB\nDATASET_LABEL\t%s\nCOLOR\tred\n' % LegendTitle)
        BinaryID_FileOut_handle.write('SHOW_LABELS\t1\nLABEL_ROTATION\t45\nLABEL_SHIFT\t5\n')
        BinaryID_FileOut_handle.write('FIELD_LABELS\t%s\n' % LegendTitle)
        BinaryID_FileOut_handle.write('FIELD_COLORS\t%s\n' % BinaryColor)
        BinaryID_FileOut_handle.write('# FIELD_SHAPES: 1: rectangle; 2: circle; 3: star; 4: right pointing triangle; 5: left pointing triangle; 6: check mark\n')
        BinaryID_FileOut_handle.write('FIELD_SHAPES\t%s\n' % BinaryShape)
        BinaryID_FileOut_handle.write('MARGIN\t10\n')
        BinaryID_FileOut_handle.write('HORIZONTAL_GRID\t0\n')
        BinaryID_FileOut_handle.write('VERTICAL_GRID\t0\n')
        BinaryID_FileOut_handle.write('\nDATA\n')
        for each_line in open(leaf_id_txt):
            leaf_id = each_line.strip().split()[0]
            BinaryID_FileOut_handle.write('%s\t1\n' % leaf_id)
        BinaryID_FileOut_handle.close()

    ####################################################################################################################

    # Prepare Heatmap file
    if Heatmap is True:

        n = 0
        col_name_list = []
        leaf_matrix_dict = {}
        for leaf_matrix in open(LeafMatrix):
            leaf_matrix_split = leaf_matrix.strip().split('\t')
            if n == 0:
                col_name_list = leaf_matrix_split[1:]
            else:
                leaf_matrix_dict[leaf_matrix_split[0]] = leaf_matrix_split[1:]
            n += 1

        Heatmap_FileOut_handle = open(FileOut, 'w')
        Heatmap_FileOut_handle.write('DATASET_HEATMAP\n')
        Heatmap_FileOut_handle.write('# Reference https://itol.embl.de/help/dataset_heatmap_template.txt\n')
        Heatmap_FileOut_handle.write('\nSEPARATOR TAB\n')
        Heatmap_FileOut_handle.write('\n# customize heatmap attributes here\n')
        Heatmap_FileOut_handle.write('MARGIN\t%s\n'          % MARGIN)
        Heatmap_FileOut_handle.write('STRIP_WIDTH\t%s\n'     % Heatmap_STRIP_WIDTH)
        Heatmap_FileOut_handle.write('\n# customize legend here\n')
        Heatmap_FileOut_handle.write('AUTO_LEGEND\t1\n')
        Heatmap_FileOut_handle.write('DATASET_LABEL\t%s\n' % LegendTitle)
        Heatmap_FileOut_handle.write('USE_MID_COLOR\t1\n')
        Heatmap_FileOut_handle.write('COLOR_MIN\t#2980B9\n')
        Heatmap_FileOut_handle.write('COLOR_MID\t#ECF0F1\n')
        Heatmap_FileOut_handle.write('COLOR_MAX\t#E74C3C\n')
        Heatmap_FileOut_handle.write('\n# customize value range here. By default, color gradients will be calculated based on dataset values\n')
        Heatmap_FileOut_handle.write('# USER_MIN_VALUE	0\n')
        Heatmap_FileOut_handle.write('# USER_MID_VALUE	5\n')
        Heatmap_FileOut_handle.write('# USER_MAX_VALUE	10\n')
        Heatmap_FileOut_handle.write('\n# customize column name here\n')
        Heatmap_FileOut_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(col_name_list))
        Heatmap_FileOut_handle.write('\n# Provide data here\n')
        Heatmap_FileOut_handle.write('DATA\n')
        for leaf in leaf_matrix_dict:
            Heatmap_FileOut_handle.write('%s\t%s\n' % (leaf, '\t'.join(leaf_matrix_dict[leaf])))
        Heatmap_FileOut_handle.close()

    ####################################################################################################################

    if Labels is True:
        if os.path.isfile(LeafLabel) is False:
            print('leaf to label file not found, please provide with -ll, program exited!')
            exit()

        # write out header
        Labels_FileOut_handle = open(FileOut, 'w')
        Labels_FileOut_handle.write('LABELS\nSEPARATOR TAB\nDATA\n')
        for each_ll in open(LeafLabel):
            Labels_FileOut_handle.write(each_ll)
        Labels_FileOut_handle.close()

    ####################################################################################################################

    if Connection is True:

        if os.path.isfile(d2r) is False:
            print('donor to recipient file not found, please provide with -dr, program exited!')
            exit()

        Connection_FileOut_handle = open(FileOut, 'w')
        Connection_FileOut_handle.write('DATASET_CONNECTION\nSEPARATOR TAB\nDATASET_LABEL\tdemo_connections\n')
        Connection_FileOut_handle.write('COLOR\t#ff0ff0\nDRAW_ARROWS\t1\nARROW_SIZE\t60\nLOOP_SIZE\t100\n')
        Connection_FileOut_handle.write('MAXIMUM_LINE_WIDTH\t10\nCURVE_ANGLE\t45\nCENTER_CURVES\t1\nALIGN_TO_LABELS\t0\nDATA\n')
        for each_connection in open(d2r):
            Connection_FileOut_handle.write(each_connection)
        Connection_FileOut_handle.close()

    ####################################################################################################################

    # Prepare PieChart file
    if PieChart is True:
        PieChart_FileOut_handle = open(FileOut, 'w')
        PieChart_FileOut_handle.write('DATASET_PIECHART\n')
        PieChart_FileOut_handle.write('SEPARATOR TAB\n')
        PieChart_FileOut_handle.write('DATASET_LABEL\t%s\n' % LegendTitle)
        PieChart_FileOut_handle.write('COLOR\t#ff0000\n')
        PieChart_FileOut_handle.write('FIELD_COLORS\t#5DADE2\t#FFFFFF\n')
        PieChart_FileOut_handle.write('FIELD_LABELS\tf1\tf2\n')
        PieChart_FileOut_handle.write('MARGIN\t10\n\n')
        PieChart_FileOut_handle.write('DATA\n')
        for each in open(LeafValue):
            each_split = each.strip().split('\t')
            node_id = each_split[0]
            node_value = float(each_split[1])
            PieChart_FileOut_handle.write('%s\t-1\t1\t%s\t%s\n' % (node_id, node_value, (100 - node_value)))
        PieChart_FileOut_handle.close()

    ####################################################################################################################

    if (ColorClade is True) or (ColorLabel is True):

        # get group_to_leaf_dict
        group_set = set()
        group_to_leaf_dict = dict()
        node_to_group_dict = dict()
        for each_line in open(LeafGroup):
            each_line_split = each_line.strip().split('\t')
            node_id    = each_line_split[0]
            node_group = each_line_split[1]
            group_set.add(node_group)
            node_to_group_dict[node_id] = node_group
            if node_group not in group_to_leaf_dict:
                group_to_leaf_dict[node_group] = [node_id]
            else:
                group_to_leaf_dict[node_group].append(node_id)

        group_list = sorted(list(group_set))

        Group_to_Color_dict = dict()

        # get groups with provided color
        Group_to_provided_Color_dict = dict()
        group_with_provided_color_list = []
        for each_group in open(GroupColor):
            each_group_split = each_group.strip().split('\t')
            group_id = each_group_split[0]
            color_code = each_group_split[1]
            if group_id in group_list:
                Group_to_provided_Color_dict[group_id] = color_code
                group_with_provided_color_list.append(group_id)

        # assign colors to the rest groups
        group_without_color_list = []
        for each_group in group_list:
            if each_group not in group_with_provided_color_list:
                group_without_color_list.append(each_group)
        if len(group_without_color_list) > 0:
            color_list_unprovided = get_color_list(len(group_without_color_list))
            Group_to_Color_dict_unprovided = dict(zip(group_without_color_list, color_list_unprovided))
            for each_group in Group_to_Color_dict_unprovided:
                Group_to_Color_dict[each_group] = Group_to_Color_dict_unprovided[each_group]

        # combine two dict
        for each_group in Group_to_provided_Color_dict:
            Group_to_Color_dict[each_group] = Group_to_provided_Color_dict[each_group]

        ColorClade_Label_FileOut_handle = open(FileOut, 'w')
        ColorClade_Label_FileOut_handle.write('TREE_COLORS\n')
        ColorClade_Label_FileOut_handle.write('SEPARATOR TAB\n')
        ColorClade_Label_FileOut_handle.write('\nDATA\n')
        for grp in group_to_leaf_dict:
            group_member = group_to_leaf_dict[grp]
            group_color = Group_to_Color_dict[grp]
            concate_str = '|'.join(group_member)
            if ColorClade is True:
                ColorClade_Label_FileOut_handle.write('%s\tclade\t%s\tnormal\t%s\n' % (concate_str, group_color, branch_width))
            if ColorLabel is True:
                ColorClade_Label_FileOut_handle.write('%s\tlabel\t%s\tnormal\t%s\n' % (concate_str, group_color, branch_width))
        ColorClade_Label_FileOut_handle.close()

    ####################################################################################################################

    # Prepare Collapse file
    if Collapse is True:

        corresponding_label_file = '%s.label.txt' % FileOut

        # read in grouping file
        group_to_leaf_dict = dict()
        for each_line in open(LeafGroup):
            each_line_split = each_line.strip().split('\t')
            leaf_id         = each_line_split[0]
            node_group      = each_line_split[1]
            if node_group not in group_to_leaf_dict:
                group_to_leaf_dict[node_group] = [leaf_id]
            else:
                group_to_leaf_dict[node_group].append(leaf_id)

        Collapse_FileOut_handle = open(FileOut, 'w')
        Collapse_FileOut_handle.write('COLLAPSE\n')
        Collapse_FileOut_handle.write('DATA\n')
        label_FileOut_handle = open(corresponding_label_file, 'w')
        label_FileOut_handle.write('LABELS\n')
        label_FileOut_handle.write('SEPARATOR TAB\n')
        label_FileOut_handle.write('DATA\n')
        for each_group in group_to_leaf_dict:
            group_member = group_to_leaf_dict[each_group]
            concate_str = '|'.join(group_member)
            Collapse_FileOut_handle.write(concate_str + '\n')
            label_FileOut_handle.write('%s\t%s\n' % (concate_str, each_group))
        Collapse_FileOut_handle.close()
        label_FileOut_handle.close()

        print('iTOL files exported to:\n%s\n%s' % (FileOut, corresponding_label_file))

    ####################################################################################################################

    # Prepare ExternalShape file
    if  MultiStyleLabel is True:
        MultiStyleLabel_FileOut_handle = open(FileOut, 'w')
        MultiStyleLabel_FileOut_handle.write('MULTI_STYLE\n')
        MultiStyleLabel_FileOut_handle.write('SEPARATOR TAB\n')
        MultiStyleLabel_FileOut_handle.write('#each line in the DATA section defines the style for one label part (specified in the first field)\n')
        MultiStyleLabel_FileOut_handle.write('\n')
        MultiStyleLabel_FileOut_handle.write('\n')
        MultiStyleLabel_FileOut_handle.write('DATA\n')
        MultiStyleLabel_FileOut_handle.close()

    ####################################################################################################################

    # Prepare ExternalShape file
    if ExternalShape is True:

        # read in leaf matrix into dict
        n = 0
        col_name_list = []
        leaf_matrix_dict = {}
        for leaf_matrix in open(LeafMatrix):
            leaf_matrix_split = leaf_matrix.strip().split('\t')
            if n == 0:
                col_name_list = leaf_matrix_split[1:]
            else:
                leaf_matrix_dict[leaf_matrix_split[0]] = leaf_matrix_split[1:]
            n += 1

        Column_to_Color_dict = {}
        if ColumnColor_txt is not None:
            for each_col in open(ColumnColor_txt):
                each_col_split = each_col.strip().split('\t')
                Column_to_Color_dict[each_col_split[0]] = each_col_split[1]

        # check if all columns in data matrix are in Column_to_Color_dict
        if ColumnColor_txt is not None:
            unfound_cols = []
            for each_col_header in col_name_list:
                if each_col_header not in Column_to_Color_dict:
                    unfound_cols.append(each_col_header)
            if len(unfound_cols) > 0:
                print('Color code for the following columns are not provided, program exited!')
                print(','.join(unfound_cols))
                exit()

        ExternalShape_FileOut_handle = open(FileOut, 'w')
        ExternalShape_FileOut_handle.write('DATASET_EXTERNALSHAPE\n')
        ExternalShape_FileOut_handle.write('# Reference https://itol.embl.de/help/dataset_external_shapes_template.txt\n')
        ExternalShape_FileOut_handle.write('\nSEPARATOR TAB\n')

        # define scale here
        if scale_str is None:
            print('Please provide scale values for ExternalShapes, e.g., 25-50-75-100, 2-4-6-8-10')
            exit()

        scale_list = scale_str.split('-')
        LEGEND_SHAPES_list = ['2'] * len(scale_list)
        LEGEND_COLORS_list = ['grey'] * len(scale_list)
        SHAPE_SCALES_list  = scale_str_to_size_list(scale_str)

        ExternalShape_FileOut_handle.write('LEGEND_TITLE\t%s\n' % LegendTitle)
        ExternalShape_FileOut_handle.write('LEGEND_SHAPES\t%s\n' % '\t'.join(LEGEND_SHAPES_list))
        ExternalShape_FileOut_handle.write('LEGEND_COLORS\t%s\n' % '\t'.join(LEGEND_COLORS_list))
        ExternalShape_FileOut_handle.write('LEGEND_LABELS\t%s\n' % '\t'.join(scale_list))
        ExternalShape_FileOut_handle.write('LEGEND_SHAPE_SCALES\t%s\n' % '\t'.join(str(i) for i in SHAPE_SCALES_list))
        ExternalShape_FileOut_handle.write('\n# customize attributes here\n')
        ExternalShape_FileOut_handle.write('VERTICAL_GRID\t0\n')
        ExternalShape_FileOut_handle.write('HORIZONTAL_GRID\t0\n')
        ExternalShape_FileOut_handle.write('SHAPE_TYPE\t2\n')
        ExternalShape_FileOut_handle.write('COLOR_FILL\t1\n')
        ExternalShape_FileOut_handle.write('SHAPE_SPACING\t1\n')
        ExternalShape_FileOut_handle.write('SHOW_LABELS\t0\n')
        ExternalShape_FileOut_handle.write('SHOW_VALUES\t0\n')
        ExternalShape_FileOut_handle.write('DASHED_LINES\t0\n')

        if LegendTitle is None:
            ExternalShape_FileOut_handle.write('DATASET_LABEL\tExternalShape\n')
        else:
            ExternalShape_FileOut_handle.write('DATASET_LABEL\t%s\n' % LegendTitle)

        # write column header/color information
        ExternalShape_FileOut_handle.write('\n# customize column header/color here\n')
        ExternalShape_FileOut_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(col_name_list))

        color_list = get_color_list(len(col_name_list))
        random.shuffle(color_list)
        if ColumnColor_txt is not None:
            color_list = [Column_to_Color_dict[i] for i in col_name_list]
        ExternalShape_FileOut_handle.write('FIELD_COLORS\t%s\n' % '\t'.join(color_list))
        ExternalShape_FileOut_handle.write('\n# Provide data here\n')
        ExternalShape_FileOut_handle.write('DATA\n')
        for leaf in leaf_matrix_dict:
            ExternalShape_FileOut_handle.write('%s\t%s\n' % (leaf, '\t'.join(leaf_matrix_dict[leaf])))

        ExternalShape_FileOut_handle.close()

    ####################################################################################################################


if __name__ == '__main__':

    iTOL_parser = argparse.ArgumentParser(usage=iTOL_usage)
    iTOL_parser.add_argument('-Labels',             required=False, action='store_true',    help='Labels')
    iTOL_parser.add_argument('-ColoredLabel',       required=False, action='store_true',    help='ColoredLabel')
    iTOL_parser.add_argument('-MultiStyleLabel',    required=False, action='store_true',    help='MultiStyleLabel')
    iTOL_parser.add_argument('-ColorStrip',         required=False, action='store_true',    help='ColorStrip')
    iTOL_parser.add_argument('-ColorRange',         required=False, action='store_true',    help='ColorRange')
    iTOL_parser.add_argument('-ColorClade',         required=False, action='store_true',    help='ColorClade')
    iTOL_parser.add_argument('-ColorLabel',         required=False, action='store_true',    help='ColorLabel')
    iTOL_parser.add_argument('-ColorLeafLabel',     required=False, action='store_true',    help='ColorLabel')
    iTOL_parser.add_argument('-SimpleBar',          required=False, action='store_true',    help='SimpleBar')
    iTOL_parser.add_argument('-Heatmap',            required=False, action='store_true',    help='Heatmap')
    iTOL_parser.add_argument('-ExternalShape',      required=False, action='store_true',    help='ExternalShape')
    iTOL_parser.add_argument('-Binary',             required=False, action='store_true',    help='Binary')
    iTOL_parser.add_argument('-BinaryID',           required=False, action='store_true',    help='Binary specified IDs as 1')
    iTOL_parser.add_argument('-BinaryShape',        required=False, default='2',            help='Binary Shape, choose from 1(rectangle), 2(circle), 3(star), 4, 5 and 6, default is 2')
    iTOL_parser.add_argument('-BinaryColor',        required=False, default='red',          help='Binary Color, default is red')
    iTOL_parser.add_argument('-Connection',         required=False, action='store_true',    help='Connection')
    iTOL_parser.add_argument('-PieChart',           required=False, action='store_true',    help='PieChart')
    iTOL_parser.add_argument('-Collapse',           required=False, action='store_true',    help='Collapse')
    iTOL_parser.add_argument('-id',                 required=False, default=None,           help='File contains leaf id')
    iTOL_parser.add_argument('-ll',                 required=False, default=None,           help='Leaf Label')
    iTOL_parser.add_argument('-lg',                 required=False, default=None,           help='Leaf Group')
    iTOL_parser.add_argument('-gc',                 required=False, default=None,           help='Specify Group/column Color (optional)')
    iTOL_parser.add_argument('-cc',                 required=False, default=None,           help='Specify Column Color (for ExternalShape format) (optional)')
    iTOL_parser.add_argument('-lv',                 required=False, default=None,           help='Leaf Value')
    iTOL_parser.add_argument('-lm',                 required=False, default=None,           help='Leaf Matrix')
    iTOL_parser.add_argument('-dr',                 required=False, default=None,           help='Donor to Recipient')
    iTOL_parser.add_argument('-scale',              required=False, default=None,           help='Scale Values, in format 0-3-6-9')
    iTOL_parser.add_argument('-lt',                 required=False, default=None,           help='Legend Title')
    iTOL_parser.add_argument('-legend',             required=False, action='store_true',    help='show legend for ColorStrip')
    iTOL_parser.add_argument('-show_strip_labels',  required=False, action='store_true',    help='SHOW_STRIP_LABELS')
    iTOL_parser.add_argument('-o',                  required=True,                          help='Output filename')
    args = vars(iTOL_parser.parse_args())
    iTOL(args)



import argparse
from ete3 import Tree


print_leaves_usage = '''
======= print_leaves example commands =======

TreeSAK print_leaves -i in.tree

=============================================
'''


def print_leaves(args):

    tree_file_in = args['i']

    leaf_list = []
    for leaf in Tree(tree_file_in, format=1):
        leaf_name = leaf.name
        leaf_list.append(leaf_name)

    print('\n'.join(sorted(leaf_list)))


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-i',      required=True,   help='input tree file')
    args = vars(parser.parse_args())
    print_leaves(args)



import os
import glob
import operator
from ete3 import Tree
from itertools import chain
from itertools import combinations


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def powerset(iterable):

    " powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3) "

    s = list(iterable)  # allows duplicate elements
    chain_obj = chain.from_iterable(combinations(s, r) for r in range(len(s)+1))
    combo_lol = []
    for _, combo in enumerate(chain_obj, 1):
        if len(list(combo)) > 0:
            combo_lol.append(list(combo))

    return combo_lol


def lca_to_two_leaves(species_tree_from_ale, internal_node_id):

    # read in ale species tree
    stree_ale = Tree(species_tree_from_ale, format=1)

    # get all leaves of the internal node
    internal_node = stree_ale.search_nodes(name=internal_node_id)[0]
    internal_node_leaf_object = internal_node.get_leaves()
    internal_node_leaf_set = set()
    for each_leaf in internal_node_leaf_object:
        internal_node_leaf_set.add(each_leaf.name)

    # get the two leaves needed
    targeted_two_leaves = []
    leaves_found = False
    for leaf_1 in internal_node_leaf_set:
        for leaf_2 in internal_node_leaf_set:
            if leaf_1 != leaf_2:
                if leaves_found is False:
                    current_lca_id = stree_ale.get_common_ancestor(leaf_1, leaf_2).name
                    if current_lca_id == internal_node_id:
                        targeted_two_leaves.append(leaf_1)
                        targeted_two_leaves.append(leaf_2)
                        leaves_found = True

    return targeted_two_leaves[0], targeted_two_leaves[1]


def keep_highest_rrtc(rrtc_in, rrtc_out):

    rrtc_highest_prob_dict = dict()
    for each_rrtc in open(rrtc_in):
        rrtc_r = each_rrtc.strip().split(':')[0].split('\t')[0]
        rrtc_d = each_rrtc.strip().split(':')[0].split('\t')[1]
        rrtc_v = float(each_rrtc.strip().split(':')[1])
        rrtc_key = '%s___%s' % (rrtc_r, rrtc_d)
        if rrtc_key not in rrtc_highest_prob_dict:
            rrtc_highest_prob_dict[rrtc_key] = rrtc_v
        else:
            if rrtc_v > rrtc_highest_prob_dict[rrtc_key]:
                rrtc_highest_prob_dict[rrtc_key] = rrtc_v

    with open(rrtc_out, 'w') as rrtc_out_handle:
        for each_rrtc in sorted(rrtc_highest_prob_dict.items(), key=operator.itemgetter(1))[::-1]:
            rrtc_r = each_rrtc[0].split('___')[0]
            rrtc_d = each_rrtc[0].split('___')[1]
            rrtc_v = each_rrtc[1]
            rrtc_out_handle.write('%s\t%s:%s\n' % (rrtc_r, rrtc_d, rrtc_v))


########################################################################################################################

# file in
ip_dir                  = '/Users/songweizhi/Desktop/DateArTree/01_HGT_ALE_with_OMA'
species_tree_from_ale   = '/Users/songweizhi/Desktop/DateArTree/05_pRTC_wd/genome_tree.newick.ufboot.ale.stree'
round_list              = [1, 2, 3, 4, 5]
color_list              = ['dodgerblue', 'goldenrod1', 'darkorange1', 'seagreen3', 'orchid3']
min_detected_times      = 2

# file out
op_dir                  = '/Users/songweizhi/Desktop/DateArTree/05_pRTC_wd/HGTs_5_rds_ALE'

########################################################################################################################

rscript                 = '%s/rscript.R'                        % op_dir
plot_file               = '%s/Venn.pdf'                         % op_dir
rtc_txt                 = '%s/rrtc.txt'                         % op_dir
rtc_txt_highest         = '%s/rrtc_uniq_by_highest_prob.txt'    % op_dir

if os.path.isdir(op_dir):
    os.system('rm -r %s' % op_dir)
os.system('mkdir %s' % op_dir)

########################################################################################################################

hgt_dict = dict()
rd_to_hgt_dict= dict()
for each_rd in round_list:

    rd_id               = each_rd
    current_rd_op_dir   = '%s/ALE4_op_dir_%s_0.3'   % (ip_dir, each_rd)
    pdf_file_re         = '%s/*.%s'                 % (current_rd_op_dir, 'pdf')
    pdf_file_list       = glob.glob(pdf_file_re)

    rd_to_hgt_dict[rd_id] = set()

    for each_pdf in pdf_file_list:
        f_path, f_base, f_ext = sep_path_basename_ext(each_pdf)
        f_base_split = f_base.split('_')
        id_by_d_to_r = '%s_to_%s' % (f_base_split[3], f_base_split[5])
        rd_og        = '%s_%s'    % (each_rd, f_base_split[0])
        rd_og_value  = '%s_%s_%s' % (each_rd, f_base_split[0], f_base_split[6])

        rd_to_hgt_dict[rd_id].add(id_by_d_to_r)

        if id_by_d_to_r not in hgt_dict:
            hgt_dict[id_by_d_to_r] = []
        hgt_dict[id_by_d_to_r].append(rd_og_value)

################################################### get Venn diagram ###################################################

combination_list = powerset(round_list)

value_str_list = []
for each_cmbo in combination_list:
    current_str = ''
    if len(each_cmbo) == 1:
        current_value = rd_to_hgt_dict[each_cmbo[0]]
        current_str = 'area%s=%s' % (each_cmbo[0], len(current_value))
        value_str_list.append(current_str)
    else:
        value_lol = []
        for each_element in each_cmbo:
            ele_value = rd_to_hgt_dict[each_element]
            value_lol.append(ele_value)
        shared = set(value_lol[0]).intersection(*value_lol)
        current_str = 'n%s=%s' % (''.join([str(i) for i in each_cmbo]), len(shared))
        value_str_list.append(current_str)

value_str     = ', '.join(value_str_list)
label_str     = '"' + '", "'.join([str(i) for i in round_list]) + '"'
color_str     = '"' + '", "'.join([str(i) for i in color_list]) + '"'
font_size_str = ', '.join(['1.2']*len(combination_list))

rscript_handle = open(rscript, 'w')
rscript_handle.write('library(futile.logger)\n')
rscript_handle.write('library(gridBase)\n')
rscript_handle.write('library(VennDiagram)\n')
rscript_handle.write('pdf(file="%s")\n' % plot_file)
rscript_handle.write('venn.plot <- draw.quintuple.venn(%s, category=c(%s), fill=c(%s), cat.col=c(%s), cat.cex=1.2, cat.dist=0.3, margin=0.05, cex=c(%s), ind=TRUE)\n' % (value_str, label_str, color_str, color_str, font_size_str))
rscript_handle.write('dev.off()\n')
rscript_handle.close()

os.system('Rscript %s' % rscript)

########################################################################################################################

rtc_txt_handle = open(rtc_txt, 'w')
qualified_hgt_num = 0
for each_hgt in hgt_dict:

    occurence_list = hgt_dict[each_hgt]
    pdf_dir = '%s/%s_%s' % (op_dir, each_hgt, len(occurence_list))
    if len(occurence_list) >= min_detected_times:

        #################### prepare rtc file ####################

        donor_id           = each_hgt.split('_to_')[0][2:]
        recipient_id       = each_hgt.split('_to_')[1][2:]
        d_leaf_1, d_leaf_2 = lca_to_two_leaves(species_tree_from_ale, donor_id)
        r_leaf_1, r_leaf_2 = lca_to_two_leaves(species_tree_from_ale, recipient_id)

        for each_occurence in occurence_list:
            value = each_occurence.split('_')[-1]
            rtc_str = '%s,%s\t%s,%s:%s' % (r_leaf_1, r_leaf_2, d_leaf_1, d_leaf_2, value)
            rtc_txt_handle.write(rtc_str + '\n')

        ##########################################################

        qualified_hgt_num += 1
        os.system('mkdir %s' % pdf_dir)
        for each_h in occurence_list:
            rd_id             = each_h.split('_')[0]
            og_id             = each_h.split('_')[1]
            value             = each_h.split('_')[2]
            pwd_input_pdf_in  = '%s/ALE4_op_dir_%s_0.3/%s_HGT_*_%s_%s.pdf'  % (ip_dir, rd_id, og_id, each_hgt,value)
            pwd_input_pdf_out = '%s/%s_%s_%s_%s.pdf'                        % (pdf_dir, rd_id, og_id, each_hgt,value)
            os.system('cp %s %s' % (pwd_input_pdf_in, pwd_input_pdf_out))
rtc_txt_handle.close()

# remove redundant HGTs, keep the one with the highest probability
keep_highest_rrtc(rtc_txt, rtc_txt_highest)

print('The number of HGTs detected in >= %s runs is %s.' % (min_detected_times, qualified_hgt_num))
print('Done!')



import io
import argparse
import arviz as az
import pandas as pd
import plotly.graph_objects as go


AssessCVG_usage = '''
================================= AssessCVG example commands =================================

TreeSAK AssessCVG -m1 r1_mcmc.txt -m2 r1_mcmc.txt -o convergence_plot.png

# This script was modified based on the script from Tianhua Liao: 
https://github.com/444thLiao/evol_tk/blob/master/dating_workflow/vis/assess_convergence.py

==============================================================================================
'''


def read_mcmc(mcmc, all_col=False):
    if type(mcmc) != str:
        return mcmc
    if all_col:
        mcmc_df = pd.read_csv(mcmc, sep='\t', index_col=0)
    else:
        f1 = open(mcmc)
        header = [_ for _ in next(f1).strip().split('\t')]
        r_header = [_ for _ in header if not _.startswith('r_g')]
        # normally it need to iterate rows and ignore the columns representing rates
        text = '\t'.join(r_header)+'\n'
        r_header=set(r_header)
        for row in f1:
            text += '\t'.join([r for r,h in zip(row.strip().split('\t'),header) if h in r_header])+'\n'
        mcmc_df = pd.read_csv(io.StringIO(text), sep='\t', index_col=0)
    return mcmc_df


def cal_HPD_CI(df,burn_in=2000):
    """
    get HPD CI through mcmc.txt directly instead of reading the log/out file.
    Only calculate high density probility 95%.
    Args:
        df (pd.DataFrame): [description]
        burn_in (int, optional): [description]. Defaults to 2000.
    """
    col2CI = {}
    for colname,col in df.iteritems():
        vals = col.values[burn_in:]
        col2CI[colname] = az.hdi(vals, hdi_prob=.95)
    return col2CI


def get_posterior_df(mcmc, burn_in=2000, scale=1, all_col=True):
    mcmc_df = read_mcmc(mcmc, all_col=all_col)
    if pd.isna(mcmc_df.iloc[-1, -1]):
        # if not completed
        mcmc_df = mcmc_df.drop(mcmc_df.index[-1])
    mcmc_df = mcmc_df.loc[~mcmc_df.isna().any(1), :]
    node_names = [_ for _ in mcmc_df.columns if _.startswith('t_n')]
    rates = [_ for _ in mcmc_df.columns if _.startswith('r_g')]
    paras = [_ for _ in mcmc_df.columns if _.startswith('mu') or _.startswith('sigma2')]

    post_df = pd.DataFrame(columns=['Posterior mean time (100 Ma)',
                                    'CI_width', 'CIs'],
                           index=node_names)
    raw_n2CI = cal_HPD_CI(mcmc_df, burn_in=burn_in)
    if 'lnL' in mcmc_df.columns:
        post_df.loc['lnL', :] = 'NA'
        post_df.loc['lnL', :] = [round(mcmc_df.loc[:, 'lnL'].mean(), 2),
                                 round(raw_n2CI['lnL'][1] - raw_n2CI['lnL'][0], 2),
                                 f"{round(raw_n2CI['lnL'][0], 2)} - {round(raw_n2CI['lnL'][1], 2)}",
                                 ]

    n2CI = {k: f"{round(v[0] * scale, 2)} - {round(v[1] * scale, 2)}"
            for k, v in raw_n2CI.items()}
    n2mean_time = {k: round(v * scale, 2)
                   for k, v in mcmc_df.mean().to_dict().items()}

    post_df.loc[node_names, 'Posterior mean time (100 Ma)'] = [n2mean_time[_]
                                                               for _ in post_df.index
                                                               if _ != 'lnL']
    post_df.loc[node_names, 'CIs'] = [n2CI[_]
                                      for _ in post_df.index
                                      if _ != 'lnL']
    post_df.loc[node_names, 'CI_width'] = [raw_n2CI[_][1] * scale - raw_n2CI[_][0] * scale
                                           for _ in post_df.index
                                           if _ != 'lnL']
    return post_df


def AssessCVG(args):

    mcmc_txt_1  = args['m1']
    mcmc_txt_2  = args['m2']
    output_plot = args['o']

    CI_1 = get_posterior_df(mcmc_txt_1)
    CI_2 = get_posterior_df(mcmc_txt_2)

    # remove lnL row
    CI_1 = CI_1.iloc[:-1, :]
    CI_2 = CI_2.iloc[:-1, :]

    dis1 = list(CI_1['Posterior mean time (100 Ma)'])
    dis2 = list(CI_2['Posterior mean time (100 Ma)'])

    fig = go.Figure()
    fig.add_scatter(x=dis1, y=dis2, name='compared', mode='markers')
    fig.add_scatter(x=[min(dis1 + dis2), max(dis1 + dis2)],
                    y=[min(dis1 + dis2), max(dis1 + dis2)],
                    mode='lines', name='y=x')

    fig.layout.width = 750
    fig.layout.height = 750
    fig.layout.xaxis.title = "run2 posterior mean time"
    fig.layout.yaxis.title = "run1 posterior mean time"
    fig.write_image(output_plot)


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-m1',      required=True,   help='mcmc.txt from run 1')
    parser.add_argument('-m2',      required=True,   help='mcmc.txt from run 2')
    parser.add_argument('-o',       required=True,   help='output convergence plot')
    args = vars(parser.parse_args())
    AssessCVG(args)



import os
import glob
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_gnm_og_cov(og_dir, og_ext, og_cov_txt):

    og_file_re   = '%s/*.%s' % (og_dir, og_ext)
    og_file_list = glob.glob(og_file_re)

    gnm_to_og_dict = dict()
    for og_file in og_file_list:
        _, _, og_id, _ = sep_path_basename_ext(og_file)
        for each_seq in SeqIO.parse(og_file, 'fasta'):
            seq_id = each_seq.id
            gnm_id = '_'.join(seq_id.split('_')[:-1])
            if gnm_id not in gnm_to_og_dict:
                gnm_to_og_dict[gnm_id] = set()
            gnm_to_og_dict[gnm_id].add(og_id)

    og_cov_txt_handle = open(og_cov_txt, 'w')
    for each_gnm in sorted(list(gnm_to_og_dict.keys())):
        gnm_og_set = gnm_to_og_dict[each_gnm]
        og_cov = len(gnm_og_set)*100/len(og_file_list)
        og_cov = float("{0:.2f}".format(og_cov))
        og_cov_txt_handle.write('%s\t%s\n' % (each_gnm, og_cov))
    og_cov_txt_handle.close()


og_dir      = '/Users/songweizhi/Desktop/OrthologousGroupsFasta_cov95'
og_ext      = 'fa'
og_cov_txt  = '/Users/songweizhi/Desktop/gnm_og_cov.txt'

get_gnm_og_cov(og_dir, og_ext, og_cov_txt)

